# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_StorageConfigurationService class.

LMI_StorageConfigurationService
-------------------------------

.. autoclass:: LMI_StorageConfigurationService
    :members:

"""

from lmi.storage.ServiceProvider import ServiceProvider
import pywbem
import blivet.formats
import lmi.providers.cmpi_logging as cmpi_logging
import lmi.storage.util.units as units
import lmi.storage.util.storage as storage
from lmi.storage.DeviceProvider import DeviceProvider
from lmi.storage.SettingProvider import SettingProvider
from lmi.providers.JobManager import Job
import time

LOG = cmpi_logging.get_logger(__name__)

class LMI_StorageConfigurationService(ServiceProvider):
    """ Provider of LMI_StorageConfigurationService. """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_StorageConfigurationService, self).__init__(
                classname="LMI_StorageConfigurationService",
                *args, **kwargs)

    @cmpi_logging.trace_method
    def _check_redundancy_setting(self, redundancy, setting):
        """
            Check that DeviceProvider.Redundancy matches Setting.
            Return None if so or string with error message if it does
            not match.
        """
        # Too many return statements, but it has a purpose.
        # pylint: disable-msg=R0911

        drmax = setting.get('DataRedundancyMax', None)
        drmin = setting.get('DataRedundancyMin', None)
        drgoal = setting.get('DataRedundancyGoal', None)

        if drmax is not None and int(drmax) < redundancy.data_redundancy:
            return "DataRedundancyMax is too low."
        if drmin is not None and int(drmin) > redundancy.data_redundancy:
            return "DataRedundancyMin is too high."
        if (drmax is None and drmin is None and drgoal is not None
                and int(drgoal) != redundancy.data_redundancy):
            # only goal is set - it must match
            return "DataRedundancyGoal does not match."

        esmax = setting.get('ExtentStripeLengthMax', None)
        esmin = setting.get('ExtentStripeLengthMin', None)
        esgoal = setting.get('ExtentStripeLength', None)
        if esmax is not None and int(esmax) < redundancy.stripe_length:
            return "ExtentStripeLengthMax is too low."
        if esmin is not None and int(esmin) > redundancy.stripe_length:
            return "ExtentStripeLengthMin is too high."
        if (esmax is None and esmin is None and esgoal is not None
                and int(esgoal) != redundancy.stripe_length):
            # only goal is set - it must match
            return "ExtentStripeLength does not match."

        prmax = setting.get('PackageRedundancyMax', None)
        prmin = setting.get('PackageRedundancyMin', None)
        prgoal = setting.get('PackageRedundancyGoal', None)
        if prmax is not None and int(prmax) < redundancy.package_redundancy:
            return "PackageRedundancyMax is too low."
        if prmin is not None and int(prmin) > redundancy.package_redundancy:
            return "PackageRedundancyMin is too high."
        if (prmax is None and prmin is None and prgoal is not None
                and prgoal != redundancy.package_redundancy):
            # only goal is set - it must match
            return "PackageRedundancyGoal does not match."

        nspof = setting.get('NoSinglePointOfFailure', None)
        if (nspof is not None
                and SettingProvider.string_to_bool(nspof)
                    != redundancy.no_single_point_of_failure):
            return "NoSinglePointOfFailure does not match."

        parity = setting.get('ParityLayout', None)
        if (parity is not None
                and int(parity) != redundancy.parity_layout):
            return "ParityLayout does not match."

        return None

    @cmpi_logging.trace_method
    def _check_redundancy_goal_setting(self, redundancy, setting):
        """
            Check that DeviceProvider.Redundancy matches Setting['*Goal'].
            Return None if so or string with error message if it does
            not match.

            If any of the *Goal property is missing, the min/max is checked
            if it fits.
        """
        drgoal = setting.get('DataRedundancyGoal', None)
        if (drgoal is not None
                and int(drgoal) != redundancy.data_redundancy):
            return "DataRedundancyGoal does not match."

        esgoal = setting.get('ExtentStripeLength', None)
        if (esgoal is not None
                and int(esgoal) != redundancy.stripe_length):
            return "ExtentStripeLength does not match."

        prgoal = setting.get('PackageRedundancyGoal', None)
        if (prgoal is not None
                and prgoal != redundancy.package_redundancy):
            return "PackageRedundancyGoal does not match."

        nspof = setting.get('NoSinglePointOfFailure', None)
        if (nspof is not None
                and SettingProvider.string_to_bool(nspof)
                    != redundancy.no_single_point_of_failure):
            return "NoSinglePointOfFailure does not match."

        parity = setting.get('ParityLayout', None)
        if (parity is not None
                and int(parity) != redundancy.parity_layout):
            return "ParityLayout does not match."

        return self._check_redundancy_setting(redundancy, setting)

    @cmpi_logging.trace_method
    def _modify_lv(self, job, devicename, name, size):
        """
            Really modify the logical volume, all parameters were checked.
        """
        device = self.storage.devicetree.getDeviceByPath(devicename)
        if device is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Device %s disappeared." % (devicename,))

        if name is not None:
            # rename
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Rename of logical volume is not yet supported.")

        if size is not None:
            # resize

            # check PE size
            newsize = device.vg.align(float(size) / units.MEGABYTE, True)
            oldsize = device.vg.align(device.size, False)
            if newsize != oldsize:
                action = blivet.deviceaction.ActionResizeDevice(
                        device, newsize)
                storage.do_storage_action(self.storage, [action])
                self.storage.devicetree.processActions(dryRun=False)
                self.storage.reset()

        newsize = device.size * units.MEGABYTE
        outparams = {}
        outparams['Size'] = pywbem.Uint64(newsize)
        devname = self.provider_manager.get_name_for_device(device)
        outparams['TheElement'] = devname
        ret = self.Values.CreateOrModifyElementFromStoragePool \
                .Job_Completed_with_No_Error

        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[devname],
                error=None)

    @cmpi_logging.trace_method
    def _create_lv(self, job, poolname, name, size):
        """
            Really create the logical volume, all parameters were checked.
        """
        pool = self.storage.devicetree.getDeviceByPath(poolname)
        if pool is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Pool %s disappeared." % (poolname,))
        args = {}
        args['parents'] = [pool]
        args['size'] = pool.align(float(size) / units.MEGABYTE, True)
        if name:
            args['name'] = name

        storage.log_storage_call("CREATE LV", args)

        lv = self.storage.newLV(**args)
        action = blivet.deviceaction.ActionCreateDevice(lv)
        storage.do_storage_action(self.storage, [action])
        # re-read the device from blivet, it should have all device links
        lv = self.storage.devicetree.getDeviceByPath(lv.path)
        newsize = lv.size * units.MEGABYTE
        lvname = self.provider_manager.get_name_for_device(lv)
        outparams = {
                'TheElement': lvname,
                'Size': pywbem.Uint64(newsize)
        }
        ret = self.Values.CreateOrModifyElementFromStoragePool \
                .Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[lvname],
                error=None)

    @cmpi_logging.trace_method
    def _parse_goal(self, param_goal, classname):
        """
            Find Setting for given CIMInstanceName and check, that it is
            of given CIM class.
            Return None, if no Goal was given.
            Raise CIMError, if the Goal cannot be found.
        """
        if param_goal:
            instance_id = param_goal['InstanceID']
            goal = self.provider_manager.get_setting_for_id(
                instance_id, classname)
            if not goal:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    classname + " Goal was not found.")
            goal.touch()
        else:
            goal = None
        return goal

    @cmpi_logging.trace_method
    def _parse_element(self, param_theelement, classname, blivet_class):
        """
            Find StorageDevice for given CIMInstanceName.
            Return None if no CIMInstanceName was given.
            Raise CIMError if the device does not exist or is not of given
            class.
        """
        if not param_theelement:
            return None
        if param_theelement.classname != classname:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Expected %s as TheElement.", (classname))

        device = self.provider_manager.get_device_for_name(param_theelement)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find the TheElement device.")
        if not isinstance(device, blivet_class):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                "The TheElement parameter is not %s." % (classname,))
        return device


    @cmpi_logging.trace_method
    def _parse_pool(self, param_inpool):
        """
            Find LVMVolumeGroupDevice for given CIMInstanceName.
            Return None if no CIMInstanceName was given.
            Raise CIMError if the device does not exist or is not VG.
        """
        if not param_inpool:
            return None

        pool = self.provider_manager.get_device_for_name(param_inpool)
        if not pool:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find the InPool device.")
        if not isinstance(pool,
            blivet.devices.LVMVolumeGroupDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "The InPool parameter is not LMI_VGStoragePool.")
        return pool

    @cmpi_logging.trace_method
    def cim_method_createormodifylv(self, env, object_name,
                                    param_elementname=None,
                                    param_goal=None,
                                    param_theelement=None,
                                    param_inpool=None,
                                    param_size=None,
                                    input_arguments=None,
                                    method_name=None):
        """
            Implements LMI_StorageConfigurationService.CreateOrModifyLV()

            Create or modify Logical Volume. This method is shortcut to
            CreateOrModifyElementFromStoragePool with the right Goal. Lazy
            applications can use this method to create or modify LVs, without
            calculation of the Goal setting.
        """
        self.check_instance(object_name)

        # remember input parameters for job
        if not input_arguments:
            input_arguments = {
                'ElementName' : pywbem.CIMProperty(name='ElementName',
                        type='string',
                        value=param_elementname),
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal),
                'TheElement': pywbem.CIMProperty(name='TheElement',
                        type='reference',
                        value=param_theelement),
                'InPool': pywbem.CIMProperty(name='InPool',
                        type='reference',
                        value=param_inpool),
                'Size': pywbem.CIMProperty(name='Size',
                        type='uint64',
                        value=param_size),
            }
        if not method_name:
            method_name = 'CreateOrModifyLV'

        # check parameters
        goal = self._parse_goal(param_goal, "LMI_LVStorageSetting")
        device = self._parse_element(param_theelement, "LMI_LVStorageExtent",
                blivet.devices.LVMLogicalVolumeDevice)
        pool = self._parse_pool(param_inpool)

        # check if resize is needed
        if param_size and device:
            oldsize = device.vg.align(device.size, False) * units.MEGABYTE
            if param_size < oldsize:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                        "Shrinking of logical volumes is not supported.")
            if oldsize == param_size:
                # don't need to change the size
                param_size = None

        # check if rename is needed
        if device and device.name == param_elementname:
            # don't need to change the name
            param_elementname = None

        # pool vs goal
        if goal and pool:
            pool_provider = self.provider_manager.get_provider_for_device(pool)
            redundancy = pool_provider.get_redundancy(pool)
            error = self._check_redundancy_setting(redundancy, goal)
            if error:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "The Goal does not match InPool's capabilities: "
                            + error)

        # pool vs theelement
        if pool and device and device.vg != pool:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                        "InPool does not match TheElement's pool, modification"\
                        " of a pool is not supported.")

        if not device and not pool:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Either InPool or TheElement must be specified.")

        if not device and not param_size:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter Size must be set when creating a logical"\
                    " volume.")

        # Schedule a job
        if device:
            jobname = "MODIFY LV %s" % (device.path)
            devname = device.path
            affected_elements = [devname]
        else:
            jobname = "CREATE LV %s FROM %s" % (param_elementname, pool.path)
            poolname = pool.path
            affected_elements = [poolname]

        job = Job(
                job_manager=self.job_manager,
                job_name=jobname,
                input_arguments=input_arguments,
                method_name=method_name,
                affected_elements=affected_elements,
                owning_element=self._get_instance_name())
        if device:
            job.set_execute_action(self._modify_lv,
                    job, devname, param_elementname, param_size)
        else:
            job.set_execute_action(self._create_lv,
                    job, poolname, param_elementname, param_size)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.CreateOrModifyLV\
                .Method_Parameters_Checked___Job_Started, outparams)

    @cmpi_logging.trace_method
    # Too many arguments of generated method: pylint: disable-msg=R0913
    def cim_method_createormodifyelementfromstoragepool(self, env, object_name,
                                                        param_elementname=None,
                                                        param_goal=None,
                                                        param_inpool=None,
                                                        param_theelement=None,
                                                        param_elementtype=None,
                                                        param_size=None):
        """
            Implements LMI_StorageConfigurationService
                            .CreateOrModifyElementFromStoragePool()

            Start a job to create (or modify) a Logical Volume from a
            LMI_StoragePool. One of the parameters for this method is Size. As
            an input parameter, Size specifies the desired size of the
            element. As an output parameter, it specifies the size achieved.
            The Size is rounded to extent size of the Volume Group. Space is
            taken from the input StoragePool. The desired settings for the
            element are specified by the Goal parameter. If the requested size
            cannot be created, no action will be taken, and the Return Value
            will be 4097/0x1001. Also, the output value of Size is set to the
            nearest possible size.  This method supports renaming or resizing
            of a Logical Volume.  If 0 is returned, the function completed
            successfully and no ConcreteJob instance was required. If
            4096/0x1000 is returned, a ConcreteJob will be started to create
            the element. The Job's reference will be returned in the output
            parameter Job.
        """
        if param_elementtype is not None:
            etype = self.Values.CreateOrModifyElementFromStoragePool.ElementType
            if param_elementtype != etype.StorageExtent:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "The only ElementType is StorageExtent (3).")

        input_arguments = {
            'ElementName' : pywbem.CIMProperty(name='ElementName',
                    type='string',
                    value=param_elementname),
            'Goal' : pywbem.CIMProperty(name='Goal',
                    type='reference',
                    value=param_goal),
            'TheElement': pywbem.CIMProperty(name='TheElement',
                    type='reference',
                    value=param_theelement),
            'InPool': pywbem.CIMProperty(name='InPool',
                    type='reference',
                    value=param_inpool),
            'ElementType': pywbem.CIMProperty(name='ElementType',
                    type='uint16',
                    value=param_elementtype),
            'Size': pywbem.CIMProperty(name='Size',
                    type='uint64',
                    value=param_size),
        }
        # create LV
        return self.cim_method_createormodifylv(env, object_name,
                param_elementname,
                param_goal,
                param_theelement,
                param_inpool,
                param_size,
                input_arguments=input_arguments,
                method_name='CreateOrModifyElementFromStoragePool')


    @cmpi_logging.trace_method
    def _modify_vg(self, job, poolname, goal, devnames, name):
        """
            Modify existing Volume Group. The parameters were already checked.
            This method is called in context of JobManager worker thread.
        """
        devices = []
        for devname in devnames:
            device = self.storage.devicetree.getDeviceByPath(devname)
            if device is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Device %s disappeared." % (devname,))
            devices.append(device)

        pool = self.provider_manager.get_device_for_name(poolname)
        if not pool:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Pool %s disappeared." % (poolname,))

        if name is not None:
            # rename
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Rename of volume group is not yet supported.")

        # check extent size
        if goal and goal['ExtentSize'] != pool.peSize:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Changing ExtentSize is not supported.")

        # check for added and removed devices:
        add_devices = []
        for device in devices:
            if device not in pool.pvs:
                add_devices.append(device)

        rm_devices = []
        for device in pool.pvs:
            if device not in devices:
                rm_devices.append(device)

        # now do it
        if add_devices:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Adding new devices to volume group is not yet supported.")
        if rm_devices:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Removing devices from volume group is not yet supported.")

        outparams = {}
        retval = self.Values.CreateOrModifyVG.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=retval,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[poolname],
                error=None)

    @cmpi_logging.trace_method
    def _create_vg(self, job, goal, devnames, name):
        """
            Create new  Volume Group. The parameters were already checked.
            This method is called in context of JobManager worker thread.
        """
        devices = []
        for devname in devnames:
            device = self.storage.devicetree.getDeviceByPath(devname)
            if device is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Device %s disappeared." % (devname,))
            devices.append(device)
        # Check the devices are unused
        storage.assert_unused(self.storage, devices)

        actions = []
        for device in devices:
            if not (device.format
                    and isinstance(device.format,
                        blivet.formats.lvmpv.LVMPhysicalVolume)):
                # create the pv format there
                pv = blivet.formats.getFormat('lvmpv')
                actions.append(blivet.ActionCreateFormat(device, pv))

        args = {}
        args['parents'] = devices
        if goal and goal['ExtentSize']:
            args['peSize'] = float(goal['ExtentSize']) / units.MEGABYTE
        if name:
            args['name'] = name

        storage.log_storage_call("CREATE VG", args)

        vg = self.storage.newVG(**args)
        actions.append(blivet.ActionCreateDevice(vg))
        storage.do_storage_action(self.storage, actions)

        poolname = self.provider_manager.get_name_for_device(vg)
        newsize = vg.size * units.MEGABYTE
        outparams = {
                'pool': poolname,
                'size':pywbem.Uint64(newsize),
        }
        retval = self.Values.CreateOrModifyVG.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=retval,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[poolname],
                error=None)

    @cmpi_logging.trace_method
    def _parse_inextents(self, param_inextents):
        """
            Find StorageDevices for given array of CIMInstanceNames and
            return couple (devices, redundancies), where devices
            is array of StorageDevices and redundancies is array
            of Redundancy of the devices.
            Return ([], []), if no InExtents were given.
            Raise CIMError, if any of the extents cannot be found.
        """
        if not param_inextents:
            return ([], [])
        devices = []
        redundancies = []
        for extent_name in param_inextents:
            provider = self.provider_manager.get_device_provider_for_name(
                    extent_name)
            if not provider:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Cannot find provider for InExtent " + str(extent_name))
            device = provider.get_device_for_name(extent_name)
            if not provider:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Cannot find device for InExtent " + str(extent_name))
            devices.append(device)
            redundancies.append(provider.get_redundancy(device))

        return (devices, redundancies)

    @cmpi_logging.trace_method
    def cim_method_createormodifyvg(self, env, object_name,
                                    param_elementname=None,
                                    param_goal=None,
                                    param_inextents=None,
                                    param_pool=None,
                                    input_arguments=None,
                                    method_name=None):
        """
            Implements LMI_StorageConfigurationService.CreateOrModifyVG()

            Create or modify Volume Group. This method is shortcut to
            CreateOrModifyStoragePool with the right Goal. Lazy applications
            can use this method to create or modify VGs, without calculation
            of the Goal setting.

            On implementation side, this method is called by
            CreateOrModifyStoragePool. If so, input_arguments and method_name
            parameters are set, so we can create proper Job here.
        """
        # check parameters
        self.check_instance(object_name)

        # remember input parameters for job
        if not input_arguments:
            input_arguments = {
                'ElementName' : pywbem.CIMProperty(name='ElementName',
                        type='string',
                        value=param_elementname),
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal),
                'InExtents': pywbem.CIMProperty(name='InExtents',
                        type='reference',
                        is_array=True,
                        value=param_inextents),
                'Pool': pywbem.CIMProperty(name='Pool',
                        type='reference',
                        value=param_pool),
            }
        if not method_name:
            method_name = 'CreateOrModifyVG'

        goal = self._parse_goal(param_goal, "LMI_VGStorageSetting")
        pool = self._parse_pool(param_pool)
        (devices, redundancies) = self._parse_inextents(param_inextents)

        # extents vs goal:
        if devices and goal:
            final_redundancy = DeviceProvider.Redundancy \
                    .get_common_redundancy_list(redundancies)
            error = self._check_redundancy_setting(final_redundancy, goal)
            if error:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "The Goal does not match InExtents' capabilities: "
                            + error)

        # elementname
        name = param_elementname
        if pool and param_elementname == pool.name:
            # no rename is needed
            name = None

        if not pool and not devices:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Either Pool or InExtents must be specified")

        # Schedule a job
        devnames = [device.path for device in devices]
        devpaths = [self.provider_manager.get_name_for_device(device)
                        for device in devices]
        if pool:
            jobname = "MODIFY VG %s" % (name)
            poolname = self.provider_manager.get_name_for_device(pool)
        else:
            # Check the devices are unused
            storage.assert_unused(self.storage, devices)
            jobname = "CREATE VG %s FROM %s" % (name, ",".join(devnames))

        job = Job(
                job_manager=self.job_manager,
                job_name=jobname,
                input_arguments=input_arguments,
                method_name=method_name,
                affected_elements=devpaths,
                owning_element=self._get_instance_name())
        if pool:
            job.set_execute_action(self._modify_vg,
                    job, poolname, goal, devnames, name)
        else:
            job.set_execute_action(self._create_vg,
                    job, goal, devnames, name)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.CreateOrModifyVG\
                .Method_Parameters_Checked___Job_Started, outparams)


    @cmpi_logging.trace_method
    # Too many aruments of generated method: pylint: disable-msg=R0913
    def cim_method_createormodifystoragepool(self, env, object_name,
                                             param_elementname=None,
                                             param_goal=None,
                                             param_inpools=None,
                                             param_inextents=None,
                                             param_pool=None,
                                             param_size=None):
        """
            Implements LMI_StorageConfigurationService.CreateOrModifyStoragePool()

            Starts a job to create (or modify) a StoragePool.Only Volume Groups
            can be created or modified using this method.\nLMI supports only
            creation of pools from whole StorageExtents, it is not possible to
            allocate only part of an StorageExtent.\nOne of the parameters for
            this method is Size. As an input parameter, Size specifies the
            desired size of the pool. It must match sum of all input extent
            sizes. Error will be returned if not, with correct Size output
            parameter value. \nAny InPools as parameter will result in
            error.\nThe capability requirements that the Pool must support are
            defined using the Goal parameter. \n This method supports renaming
            of a Volume Group and adding and removing StorageExtents to/from a
            Volume Group. \nIf 0 is returned, then the task completed
            successfully and the use of ConcreteJob was not required. If the
            task will take some time to complete, a ConcreteJob will be
            created and its reference returned in the output parameter Job. \n
            This method automatically formats the StorageExtents added to a
            Volume Group as Physical Volumes.
        """
        # check parameters
        if param_size:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter Size is not supported.")
        if param_inpools:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter InPools is not supported.")

        input_arguments = {
            'ElementName' : pywbem.CIMProperty(name='ElementName',
                    type='string',
                    value=param_elementname),
            'Goal' : pywbem.CIMProperty(name='Goal',
                    type='reference',
                    value=param_goal),
            'InExtents': pywbem.CIMProperty(name='InExtents',
                    type='reference',
                    is_array=True,
                    value=param_inextents),
            'InPools': pywbem.CIMProperty(name='InPools',
                    type='reference',
                    is_array=True,
                    value=param_inpools),
            'Pool': pywbem.CIMProperty(name='Pool',
                    type='reference',
                    value=param_pool),
            'Size': pywbem.CIMProperty(name='Size',
                    type='uint64',
                    value=param_size),
        }

        return self.cim_method_createormodifyvg(env, object_name,
                param_elementname, param_goal, param_inextents, param_pool,
                input_arguments, 'CreateOrModifyStoragePool')

    @cmpi_logging.trace_method
    # Too many aruments of generated method: pylint: disable-msg=R0913
    def cim_method_createormodifyelementfromelements(self, env, object_name,
                                                     param_inelements,
                                                     param_elementtype,
                                                     param_elementname=None,
                                                     param_goal=None,
                                                     param_theelement=None,
                                                     param_size=None):
        """
            Implements LMI_StorageConfigurationService.CreateOrModifyElementFromElements()

            Start a job to create (or modify) a MD RAID from specified input
            StorageExtents. Only whole StorageExtents can be added to a
            RAID.\nAs an input parameter, Size specifies the desired size of
            the element and must match size of all input StorageVolumes
            combined in the RAID. Use null to avoid this calculation. As an
            output parameter, it specifies the size achieved. \n The desired
            Settings for the element are specified by the Goal parameter. \n
            If 0 is returned, the function completed successfully and no
            ConcreteJob instance was required. If 4096/0x1000 is returned, a
            ConcreteJob will be started to create the element. The Job\'s
            reference will be returned in the output parameter Job.\n This
            method does not support MD RAID modification for now.
        """
        # check parameters
        if param_size is not None:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter Size is not supported.")

        if param_elementtype is not None:
            etypes = self.Values.CreateOrModifyElementFromElements.ElementType
            if param_elementtype != etypes.Storage_Extent:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                        "Parameter ElementType must have value" \
                        " '3 - StorageExtent'.")
        input_arguments = {
                'InElements': pywbem.CIMProperty(name='InElements',
                        type='reference',
                        is_array=True,
                        value=param_inelements),
                'ElementType': pywbem.CIMProperty(name='ElementType',
                        type='int16',
                        value=param_elementtype),
                'ElementName' : pywbem.CIMProperty(name='ElementName',
                        type='string',
                        value=param_elementname),
                'TheElement': pywbem.CIMProperty(name='TheElement',
                        type='reference',
                        value=param_theelement),
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal),
                'Size' : pywbem.CIMProperty(name='Size',
                        type='uint64',
                        value=param_size),
        }

        return self.cim_method_createormodifymdraid(env, object_name,
                param_elementname=param_elementname,
                param_theelement=param_theelement,
                param_goal=param_goal,
                param_level=None,
                param_inextents=param_inelements,
                input_arguments=input_arguments,
                method_name='CreateOrModifyElementFromElements')

    @cmpi_logging.trace_method
    def _find_raid_level(self, redundancies, goal):
        """
           Find and return RAID level corresponding to given Goal and set of
           redundancies of input devices.
        """
        # find redundancies of the devices
        # try all possible RAID levels and take the best one
        # hash level -> redundancy of the RAID with given level
        levels = {
                0: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 0),
                1: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 1),
                4: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 4),
                5: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 5),
                6: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 6),
                10: DeviceProvider.Redundancy.get_common_redundancy_list(
                        redundancies, 10),
        }

        # hash RAID level number -> priority of the level
        # if more RAID levels matches the goal, the lowest priority is selected
        level_priorities = {
                1: 1,
                5: 2,
                6: 3,
                4: 4,
                10: 5,
                0: 6
        }

        # first, check the goal[*Goal] properties
        best_level = None
        for (level, redundancy) in levels.iteritems():
            err = self._check_redundancy_goal_setting(redundancy, goal)
            if err is None:
                # we have match which either completely satisfied goal[*Goal]
                # or the redundancy matches goal[*Min/Max] properties
                if not best_level:
                    best_level = level
                else:
                    if level_priorities[level] < level_priorities[best_level]:
                        best_level = level
                LOG().trace_info(
                        "Goal check: matching RAID%d, best level so far: %d",
                        level, best_level)
            else:
                LOG().trace_info("Goal check: skipping goal RAID%d: %s",
                            level, err)

        if best_level is not None:
            return best_level

        # then, find any that matches
        for (level, redundancy) in levels.iteritems():
            err = self._check_redundancy_setting(redundancy, goal)
            if err is None:
                if not best_level:
                    best_level = level
                else:
                    if level_priorities[level] < level_priorities[best_level]:
                        best_level = level
                LOG().trace_info(
                        "Any check: matching RAID%d, best level so far: %d",
                        level, best_level)
            else:
                LOG().trace_info("Any check: skipping RAID%d: %s", level, err)
        return best_level

    @cmpi_logging.trace_method
    def _schedule_create_mdraid(self, level, goal, devices, name,
            input_arguments, method_name):
        """
        Create the job to create a MD RAID.
        """
        devnames = [device.path for device in devices]
        devpaths = [self.provider_manager.get_name_for_device(device)
                        for device in devices]
        job = Job(
                job_manager=self.job_manager,
                job_name="CREATE MDRAID ON " + "+".join(devnames),
                input_arguments=input_arguments,
                method_name=method_name,
                affected_elements=devpaths,
                owning_element=self._get_instance_name())
        job.set_execute_action(self._create_mdraid,
                job, level, goal, devnames, name)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.CreateOrModifyMDRAID\
                .Method_Parameters_Checked___Job_Started, outparams)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def _create_mdraid(self, job, level, goal, devicenames, name):
        """
            Create new  MD RAID. The parameters were already checked.
        """
        # Delete format on all devices first
        # This is workaround for #924245
        # TODO: remove when the bug is fixed
        actions = []
        for devname in devicenames:
            device = self.storage.devicetree.getDeviceByPath(devname)
            if device:
                actions.append(blivet.ActionDestroyFormat(device))
        storage.do_storage_action(self.storage, actions)
        # End of workaround


        # covert devices from strings to real devices
        devices = []
        for devname in devicenames:
            device = self.storage.devicetree.getDeviceByPath(devname)
            if device is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Device %s disappeared." % (devname,))
            devices.append(device)
        # Check the devices are unused
        storage.assert_unused(self.storage, devices)

        args = {}
        args['parents'] = devices
        if name:
            args['name'] = name
        args['level'] = str(level)
        args['memberDevices'] = len(devices)

        storage.log_storage_call("CREATE MDRAID", args)

        raid = self.storage.newMDArray(**args)
        action = blivet.ActionCreateDevice(raid)
        storage.do_storage_action(self.storage, [action])

        # re-read the device from blivet, it should have all device links
        raid = self.storage.devicetree.getDeviceByPath(raid.path)
        newsize = raid.size * units.MEGABYTE
        raidname = self.provider_manager.get_name_for_device(raid)
        outparams = {
            'theelement': raidname,
            'size' : pywbem.Uint64(newsize)
        }
        retval = self.Values.CreateOrModifyMDRAID.Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=retval,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[raidname, ],
                error=None)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def _schedule_modify_mdraid(self, raid, level, goal, devices, name,
            input_parameters, method_name):
        """
            Modify existing MD RAID. The parameters were already checked.
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "MD RAID modification is not supported yet.")

    @cmpi_logging.trace_method
    def cim_method_createormodifymdraid(self, env, object_name,
                                        param_elementname=None,
                                        param_theelement=None,
                                        param_goal=None,
                                        param_level=None,
                                        param_inextents=None,
                                        input_arguments=None,
                                        method_name=None):
        """
            Implements LMI_StorageConfigurationService.CreateOrModifyMDRAID()

            Create or modify MD RAID array. This method is shortcut to
            CreateOrModifyElementFromElements with the right Goal. Lazy
            applications can use this method to create or modify MD RAID with
            the right level, without calculation of the Goal setting.\n Either
            Level or Goal must be specified. If both are specified, they must
            match.\n RAID modification is not yet supported.
        """
        # check parameters
        self.check_instance(object_name)

        # remember input parameters for job
        if not input_arguments:
            input_arguments = {
                    'ElementName' : pywbem.CIMProperty(name='ElementName',
                            type='string',
                            value=param_elementname),
                    'TheElement': pywbem.CIMProperty(name='TheElement',
                            type='reference',
                            value=param_theelement),
                    'Goal' : pywbem.CIMProperty(name='Goal',
                            type='reference',
                            value=param_goal),
                    'Level' : pywbem.CIMProperty(name='Level',
                            type='uint16',
                            value=param_level),
                    'InExtents': pywbem.CIMProperty(name='InExtents',
                            type='reference',
                            is_array=True,
                            value=param_inextents),
            }
        if not method_name:
            method_name = "CreateOrModifyMDRAID"

        # check the parameters
        goal = self._parse_goal(param_goal, "LMI_MDRAIDStorageSetting")
        raid = self._parse_element(param_theelement, "LMI_MDRAIDStorageExtent",
                blivet.devices.MDRaidArrayDevice)
        (devices, redundancies) = self._parse_inextents(param_inextents)

        # level
        if param_level is not None and param_level not in (
                self.Values.CreateOrModifyMDRAID.Level.RAID0,
                self.Values.CreateOrModifyMDRAID.Level.RAID1,
                self.Values.CreateOrModifyMDRAID.Level.RAID4,
                self.Values.CreateOrModifyMDRAID.Level.RAID5,
                self.Values.CreateOrModifyMDRAID.Level.RAID6,
                self.Values.CreateOrModifyMDRAID.Level.RAID10):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Invalid value of parameter Level.")

        # goal vs level
        if goal and param_level is not None:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Only one of Level and Goal parameters may be used.")

        # extents vs goal:
        if devices and goal:
            # guess RAID level
            param_level = self._find_raid_level(redundancies, goal)
            if param_level is None:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "The Goal does not match any RAID level for InExtents.")

        # nr. of devices vs level
        if ((param_level == 0
                or param_level == 1)
                    and len(devices) < 2):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "At least two devices are required for RAID level %d."
                    % (param_level))

        if (param_level == 5 or param_level == 4) and len(devices) < 3:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "At least three devices are required for RAID level" \
                    " 4 or 5.")
        if param_level == 6 and len(devices) < 4:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "At least four devices are required for RAID level 6.")
        if param_level == 10 and len(devices) < 2:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "At least two devices are required for RAID level 10.")

        name = param_elementname
        if raid and param_elementname == raid.name:
            # no rename is needed
            name = None

        if not raid and not devices:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Either TheElement or InExtents must be specified")

        if raid:
            return self._schedule_modify_mdraid(
                    raid, param_level, goal, devices, name, input_arguments,
                    method_name)
        else:
            # Check the devices are unused
            storage.assert_unused(self.storage, devices)
            return self._schedule_create_mdraid(
                    param_level, goal, devices, name, input_arguments,
                    method_name)

    @cmpi_logging.trace_method
    def _delete_mdraid(self, job, devicepath):
        """
        Really delete a MD RAID array. This method is called in context of
        JobManager worker thread.
        """
        device = self.storage.devicetree.getDeviceByPath(devicepath)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find device %s" % (devicepath,))
        if not isinstance(device, blivet.devices.MDRaidArrayDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Device %s is not LMI_MDRAIDStorageExtent" % (devicepath,))

        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # finally delete it
        actions = []
        actions.append(blivet.ActionDestroyDevice(device))
        # Destroy also all formats on member devices.
        # TODO: remove when Blivet does this automatically.
        for parent in device.parents:
            actions.append(blivet.deviceaction.ActionDestroyFormat(parent))
        storage.do_storage_action(self.storage, actions)

        ret = self.Values.DeleteMDRAID.Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments={},
                affected_elements=[],
                error=None)

    @cmpi_logging.trace_method
    def cim_method_deletemdraid(self, env, object_name,
                                param_theelement=None):
        """Implements LMI_StorageConfigurationService.DeleteMDRAID()

        Delete MD RAID array. All members are detached from the array and
        all RAID metadata are erased.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
            specifying the object on which the method DeleteMDRAID()
            should be invoked.
        param_theelement --  The input parameter TheElement (type REF (pywbem.CIMInstanceName(classname='LMI_MDRAIDStorageExtent', ...))
            The MD RAID device to destroy.


        Returns a two-tuple containing the return value (type pywbem.Uint32 self.Values.DeleteMDRAID)
        and a list of CIMParameter objects representing the output parameters

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...))
            Reference to the job (may be null if job completed).


        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate,
            unrecognized or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the target CIM Class or instance does not
            exist in the specified namespace)
        CIM_ERR_METHOD_NOT_AVAILABLE (the CIM Server is unable to honor
            the invocation request)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # check parameters
        self.check_instance(object_name)

        # remember input parameters for job
        input_arguments = {
                'TheElement': pywbem.CIMProperty(name='TheElement',
                        type='reference',
                        value=param_theelement),
        }

        # check the parameters
        raid = self._parse_element(param_theelement, "LMI_MDRAIDStorageExtent",
                blivet.devices.MDRaidArrayDevice)
        if not raid:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter TheElement must be specified an must be "
                    "reference to LMI_MDRAIDStorageExtent")
        # Check the device is unused
        storage.assert_unused(self.storage, [raid])

        # Schedule the job
        job = Job(
                job_manager=self.job_manager,
                job_name="DELETE MDRAID " + raid.path,
                input_arguments=input_arguments,
                method_name='DeleteMDRAID',
                affected_elements=[param_theelement],
                owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_mdraid,
                job, raid.path)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.CreateOrModifyMDRAID\
                .Method_Parameters_Checked___Job_Started, outparams)


    @cmpi_logging.trace_method
    def _delete_vg(self, job, poolpath):
        """
        Delete VG. This method is called in context of
        JobManager worker thread.
        """
        pool = self.storage.devicetree.getDeviceByPath(poolpath)
        if not pool:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find VG %s" % (poolpath,))
        if not isinstance(pool, blivet.devices.LVMVolumeGroupDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Device %s is not LMI_VGStoragePool" % (poolpath,))

        # Check the device is unused
        storage.assert_unused(self.storage, [pool])

        # finally delete it
        action = blivet.ActionDestroyDevice(pool)
        storage.do_storage_action(self.storage, [action])

        ret = self.Values.DeleteVG.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments={},
                affected_elements=[],
                error=None)

    @cmpi_logging.trace_method
    def cim_method_deletevg(self, env, object_name,
                            param_pool=None, input_arguments=None,
                            method_name=None):
        """Implements LMI_StorageConfigurationService.DeleteVG()

        Start a job to delete a Volume Group. If 0 is returned, the
        function completed successfully, and no ConcreteJob was required.
        If 4096/0x1000 is returned, a ConcreteJob will be started to
        delete the StoragePool. A reference to the Job is returned in the
        Job parameter.
        """
        self.check_instance(object_name)
        # remember input parameters for job
        if not input_arguments:
            input_arguments = {
                    'Pool': pywbem.CIMProperty(name='Pool',
                            type='reference',
                            value=param_pool),
            }
        if not method_name:
            method_name = "DeleteVG"

        # check the parameters
        pool = self._parse_pool(param_pool)
        if not pool:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Parameter Pool is mandatory and must be"
                    " LMI_VGStoragePool.")
        # Check the device is unused
        storage.assert_unused(self.storage, [pool])

        # Schedule the job
        job = Job(
                job_manager=self.job_manager,
                job_name="DELETE VG " + pool.path,
                input_arguments=input_arguments,
                method_name=method_name,
                affected_elements=[param_pool],
                owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_vg,
                job, pool.path)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.DeleteVG\
                .Method_Parameters_Checked___Job_Started, outparams)

    @cmpi_logging.trace_method
    def cim_method_deletestoragepool(self, env, object_name,
                                     param_pool=None):
        """Implements LMI_StorageConfigurationService.DeleteStoragePool()

        Start a job to delete a StoragePool. The freed space is returned
        source StoragePools (indicated by AllocatedFrom StoragePool) or
        back to underlying storage extents. If 0 is returned, the function
        completed successfully, and no ConcreteJob was required. If
        4096/0x1000 is returned, a ConcreteJob will be started to delete
        the StoragePool. A reference to the Job is returned in the Job
        parameter.

        Implementation just calls DeleteVG with the same arguments.
        """
        input_arguments = {
                'Pool': pywbem.CIMProperty(name='Pool',
                        type='reference',
                        value=param_pool),
        }
        return self.cim_method_deletevg(env, object_name, param_pool,
                input_arguments, "DeleteStoragePool")

    @cmpi_logging.trace_method
    def _delete_lv(self, job, devicepath):
        """
        Delete VG. This method is called in context of
        JobManager worker thread.
        """
        device = self.storage.devicetree.getDeviceByPath(devicepath)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find LV %s" % (devicepath,))
        if not isinstance(device, blivet.devices.LVMLogicalVolumeDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Device %s is not LMI_LVStorageExtent" % (devicepath,))

        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # finally delete it
        action = blivet.ActionDestroyDevice(device)
        storage.do_storage_action(self.storage, [action])

        ret = self.Values.DeleteLV.Job_Completed_with_No_Error
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments={},
                affected_elements=[],
                error=None)

    @cmpi_logging.trace_method
    def cim_method_deletelv(self, env, object_name,
                            param_theelement=None,
                            input_arguments=None, method_name=None):
        """Implements LMI_StorageConfigurationService.DeleteLV()

        Start a job to delete a  Logical Volume. If 0 is returned, the
        function completed successfully and no ConcreteJob was required.
        If 4096/0x1000 is returned, a ConcreteJob will be started to
        delete the element. A reference to the Job is returned in the Job
        parameter. This method is alias of ReturnToStoragePool().
        """

        self.check_instance(object_name)
        # remember input parameters for job
        if not input_arguments:
            input_arguments = {
                    'TheElement': pywbem.CIMProperty(name='TheElement',
                            type='reference',
                            value=param_theelement),
            }
        if not method_name:
            method_name = "DeleteLV"

        # check the parameters
        device = self._parse_element(param_theelement, "LMI_LVStorageExtent",
                blivet.devices.LVMLogicalVolumeDevice)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Parameter TheElement is mandatory and must be"
                    " LMI_LVStorageExtent.")
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # Schedule the job
        job = Job(
                job_manager=self.job_manager,
                job_name="DELETE LV " + device.path,
                input_arguments=input_arguments,
                method_name=method_name,
                affected_elements=[param_theelement],
                owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_lv,
                job, device.path)

        # enqueue the job
        self.job_manager.add_job(job)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        return (self.Values.DeleteLV\
                .Method_Parameters_Checked___Job_Started, outparams)

    @cmpi_logging.trace_method
    def cim_method_returntostoragepool(self, env, object_name,
                                       param_theelement=None):
        """Implements LMI_StorageConfigurationService.ReturnToStoragePool()

        Start a job to delete an element, i.e. Logical Volume, previously
        created from a StoragePool. The freed space is returned to the
        source StoragePool. If 0 is returned, the function completed
        successfully and no ConcreteJob was required. If 4096/0x1000 is
        returned, a ConcreteJob will be started to delete the element. A
        reference to the Job is returned in the Job parameter.
        """
        input_arguments = {
                'TheElement': pywbem.CIMProperty(name='TheElement',
                        type='reference',
                        value=param_theelement),
        }
        return self.cim_method_deletelv(env, object_name,
                param_theelement, input_arguments, "ReturnToStoragePool")


    class Values(ServiceProvider.Values):
        class CreateOrModifyElementFromStoragePool(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535
            class ElementType(object):
                Unknown = pywbem.Uint16(0)
                Reserved = pywbem.Uint16(1)
                StorageVolume = pywbem.Uint16(2)
                StorageExtent = pywbem.Uint16(3)
                LogicalDisk = pywbem.Uint16(4)
                ThinlyProvisionedStorageVolume = pywbem.Uint16(5)
                ThinlyProvisionedLogicalDisk = pywbem.Uint16(6)
                # DMTF_Reserved = ..
                # Vendor_Specific = 32768..65535

        class CreateOrModifyLV(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535

        class CreateOrModifyVG(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)

        class CreateOrModifyStoragePool(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535

        class CreateOrModifyElementFromElements(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535
            class ElementType(object):
                Unknown = pywbem.Uint16(0)
                Reserved = pywbem.Uint16(1)
                Storage_Volume = pywbem.Uint16(2)
                Storage_Extent = pywbem.Uint16(3)
                Storage_Pool = pywbem.Uint16(4)
                Logical_Disk = pywbem.Uint16(5)
                ThinlyProvisionedStorageVolume = pywbem.Uint16(6)
                ThinlyProvisionedLogicalDisk = pywbem.Uint16(7)
                # DMTF_Reserved = ..
                # Vendor_Specific = 32768..65535

        class CreateOrModifyMDRAID(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535
            class Level(object):
                RAID0 = pywbem.Uint16(0)
                RAID1 = pywbem.Uint16(1)
                RAID4 = pywbem.Uint16(4)
                RAID5 = pywbem.Uint16(5)
                RAID6 = pywbem.Uint16(6)
                RAID10 = pywbem.Uint16(10)
        class DeleteMDRAID(object):
            Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535
        class DeleteVG(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            # Method_Reserved = 4097..32767
            # Vendor_Specific = 32768..65535
        class DeleteLV(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            # Method_Reserved = 4097..32767
            # Vendor_Specific = 32768..65535



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-

"""
Module for BaseProvider class.

BaseProvider
------------

.. autoclass:: BaseProvider
    :members:

"""

from pywbem.cim_provider2 import CIMProvider2
import lmi.providers.cmpi_logging as cmpi_logging

class BaseProvider(CIMProvider2):
    """
        CIM Provider for LMI. It adds access to ProviderManager, configuration
        and blivet.Blivet instance.

        In addition to CIM provider methods, this class and its subclasses
        can convert CIM InstanceName to Anaconda's StorageDevice instance
        and a vice versa.
    """
    @cmpi_logging.trace_method
    def __init__(self, storage, config, provider_manager, setting_manager,
            job_manager, *args, **kwargs):
        """
            Initialize the provider.
            Store reference to blivet.Blivet.
            Store reference to StorageConfiguration.
            Register at given ProviderManager.
        """
        super(BaseProvider, self).__init__(*args, **kwargs)
        self.storage = storage
        self.config = config
        self.provider_manager = provider_manager
        self.setting_manager = setting_manager
        self.job_manager = job_manager

    @cmpi_logging.trace_method
    # The method has too many arguments, but that's because of
    # CIMProvider2.references
    # pylint: disable-msg=R0913
    def simple_references(self, env, object_name, model, result_class_name,
            role, result_role, keys_only, first_class, second_class):
        """
            Implementation of basic associations. This method can be used by
            all subclasses, which implement association provider.

            It has the same arguments as references() method of CIMProvider2,
            additional 'first_class' and 'second_class' should contain
            string with CIM class names of the associated classes.

            All four association-related operations (Associators, AssociatorNames,
            References, ReferenceNames) are mapped to this method.
            This method is a python generator

            Keyword arguments:
            env -- Provider Environment (pycimmb.ProviderEnvironment)
            object_name -- A pywbem.CIMInstanceName that defines the source
                CIM Object whose associated Objects are to be returned.
            model -- A template pywbem.CIMInstance to serve as a model
                of the objects to be returned.  Only properties present on this
                model need to be set.
            result_class_name -- If not empty, this string acts as a filter on
                the returned set of Instances by mandating that each returned
                Instances MUST represent an association between object_name
                and an Instance of a Class whose name matches this parameter
                or a subclass.
            role -- If not empty, MUST be a valid Property name. It acts as a
                filter on the returned set of Instances by mandating that each
                returned Instance MUST refer to object_name via a Property
                whose name matches the value of this parameter.
            result_role -- If not empty, MUST be a valid Property name. It acts
                as a filter on the returned set of Instances by mandating that
                each returned Instance MUST represent associations of
                object_name to other Instances, where the other Instances play
                the specified result_role in the association (i.e. the
                name of the Property in the Association Class that refers to
                the Object related to object_name MUST match the value of this
                parameter).
            keys_only -- A boolean.  True if only the key properties should be
                set on the generated instances.

            The following diagram may be helpful in understanding the role,
            result_role, and result_class_name parameters.
            +------------------------+                    +-------------------+
            | object_name.classname  |                    | result_class_name |
            | ~~~~~~~~~~~~~~~~~~~~~  |                    | ~~~~~~~~~~~~~~~~~ |
            +------------------------+                    +-------------------+
               |              +-----------------------------------+      |
               |              |  [Association] model.classname    |      |
               | object_name  |  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~    |      |
               +--------------+ object_name.classname REF role    |      |
            (CIMInstanceName) | result_class_name REF result_role +------+
                              |                                   |(CIMInstanceName)
                              +-----------------------------------+

            Possible Errors:
            CIM_ERR_ACCESS_DENIED
            CIM_ERR_NOT_SUPPORTED
            CIM_ERR_INVALID_NAMESPACE
            CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
                or otherwise incorrect parameters)
            CIM_ERR_FAILED (some other unspecified error occurred)
        """

        cimom = env.get_cimom_handle()
        if cimom.is_subclass(object_name.namespace,
                  sub=object_name.classname,
                  super=first_class) or \
                  cimom.is_subclass(object_name.namespace,
                       sub=object_name.classname,
                       super=second_class):
            return self.simple_refs(env, object_name, model,
                      result_class_name, role, result_role, keys_only)





# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for CapabilitiesProvider class.

CapabilitiesProvider
--------------------

.. autoclass:: CapabilitiesProvider
    :members:

ElementCapabilitiesProvider
---------------------------

.. autoclass:: ElementCapabilitiesProvider
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.ComputerSystem import get_system_name

class CapabilitiesProvider(BaseProvider):
    """
        Base class for every LMI_*Capabilities providers.
        It implements get_instance and enum_instances methods.

        This class assumes that most LMI_*Capabilities are associated to
        appropriate LMI_*Service and adds support for it.

        Of course, LMI_*Capabilities can be associated to different instances
        in subclasses of this provider.
    """

    DEFAULT_CAPABILITY = "_default"

    @cmpi_logging.trace_method
    def __init__(self, classname, *args, **kwargs):
        super(CapabilitiesProvider, self).__init__(*args, **kwargs)
        self.classname = classname

    @cmpi_logging.trace_method
    def create_capabilities_id(self, myid):
        """
            InstanceID should have format LMI:<classname>:<ID>.
            This method returns string LMI:<classname>:<myid>
        """
        return "LMI:" + self.classname + ":" + myid

    @cmpi_logging.trace_method
    def get_capabilities_for_id(self, instance_id):
        """
            Return dictionary property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
            Return None if there is no such Capabilities instance.

            Subclasses can override this method.
        """
        for capabilities in self.enumerate_capabilities():
            if capabilities['InstanceID'] == instance_id:
                return capabilities
        return None

    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.

            Subclasses must override this method.
        """
        return []

    # pylint: disable-msg=W0613
    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        """
            Create LMI_*Setting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.

            Subclasses must override this method.
        """
        return None

    # pylint: disable-msg=W0221
    @cmpi_logging.trace_method
    def get_instance(self, env, model, capabilities=None):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        if not capabilities:
            instance_id = model['InstanceID']
            capabilities = self.get_capabilities_for_id(instance_id)
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Capabilities not found.")

        # skip _default and other underscore properties
        for (key, value) in capabilities.iteritems():
            if key.startswith("_"):
                continue
            model[key] = value

        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'InstanceID': None})

        for capabilities in self.enumerate_capabilities():
            model['InstanceID'] = capabilities['InstanceID']
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, capabilities)

    @cmpi_logging.trace_method
    def is_default(self, capabilities):
        """
            Return True, if the capabilities are the default one, i.e.
            with ElementCapabilities.Characteristics == Default.
        """
        return capabilities.has_key(self.DEFAULT_CAPABILITY)

    @cmpi_logging.trace_method
    def cim_method_createsetting(self, env, object_name):
        """Implements LMI_*Capabilities.CreateSetting()

        Create LMI_*Setting according to this capabilities.
        All properties its will have default values.
        """
        capabilities = self.get_capabilities_for_id(object_name['InstanceID'])
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Capabilities not found.")

        setting_name = self.create_setting_for_capabilities(capabilities)

        if not setting_name:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Unknown error when creating setting.")
        out_params = [pywbem.CIMParameter('setting', type='reference',
                value=setting_name)]
        rval = self.Values.CreateSetting.Success
        return (rval, out_params)

    @cmpi_logging.trace_method
    def cim_method_lmi_createsetting(self, env, object_name):
        self.cim_method_createsetting(env, object_name)

    @cmpi_logging.trace_method
    def get_default_capabilities(self):
        """
            Return default capabilities or None if there are no default ones.
        """
        for capabilities in self.enumerate_capabilities():
            if self.is_default(capabilities):
                return capabilities
        return None

    @cmpi_logging.trace_method
    def get_name_for_id(self, instance_id):
        """ Return CIMInstanceName for given InstanceID. """
        return pywbem.CIMInstanceName(
                classname=self.classname,
                namespace=self.config.namespace,
                keybindings={"InstanceID": instance_id})


    class Values(object):
        class CreateSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Failed = pywbem.Uint32(4)

class ElementCapabilitiesProvider(BaseProvider):
    """
        Base class for LMI_*ElementCapabilities providers.

        If all capabilities instances are associated only with appropriate
        LMI_*Service, this class does not need to be subclasses.

        Otherwise, subclasses can associate capabilities to other managed
        elements.
    """
    @cmpi_logging.trace_method
    def __init__(self, classname, capabilities_provider, service_provider,
            *args, **kwargs):
        self.classname = classname
        self.capabilities_provider = capabilities_provider
        self.service_provider = service_provider
        super(ElementCapabilitiesProvider, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return iterable with (managed_element_name, capabilities_name),
            where managed_element_name and capabilities_name
            are CIMInstanceName.

            By default, all capabilities provided by capabilities_provider
            are associated to service_provider.

            Subclasses can override this method if different behavior is
            requested.
        """
        for capabilities in self.capabilities_provider.enumerate_capabilities():
            managed_element_name = pywbem.CIMInstanceName(
                    classname=self.service_provider.classname,
                    namespace=self.config.namespace,
                    keybindings={
                            'CreationClassName':
                                    self.service_provider.classname,
                            'Name': self.service_provider.classname,
                            'SystemCreationClassName' :
                                    self.config.system_class_name,
                            'SystemName': get_system_name()
                    })

            capabilities_name = pywbem.CIMInstanceName(
                    classname=self.capabilities_provider.classname,
                    namespace=self.config.namespace,
                    keybindings={'InstanceID' : capabilities['InstanceID']})
            yield (managed_element_name, capabilities_name)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'Capabilities': None, 'ManagedElement': None})
        for (managed_element, capabilities) in self.enumerate_capabilities():
            model['Capabilities'] = capabilities
            model['ManagedElement'] = managed_element
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, capabilities)

    # pylint: disable-msg=W0221
    @cmpi_logging.trace_method
    def get_instance(self, env, model, capabilities=None):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        # find the capabilities instance
        if not capabilities:
            for (element_name, capabilities) in self.enumerate_capabilities():
                if (element_name == model['ManagedElement']
                        and capabilities == model['Capabilities']):
                    break
            else:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "ElementCapabilities not found.")

        capabilities = self.capabilities_provider.get_capabilities_for_id(
                capabilities['InstanceID'])

        characteristics = [self.Values.Characteristics.Current]
        if self.capabilities_provider.is_default(capabilities):
            characteristics.append(self.Values.Characteristics.Default)

        model['Characteristics'] = characteristics
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations. """
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_Capabilities",
                "CIM_ManagedElement")


    class Values(object):
        class Characteristics(object):
            Default = pywbem.Uint16(2)
            Current = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_FileSystemConfigurationCapabilities class.

LMI_FileSystemConfigurationCapabilities
---------------------------------------

.. autoclass:: LMI_FileSystemConfigurationCapabilities
    :members:

"""

from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_FileSystemConfigurationCapabilities(CapabilitiesProvider):
    """
        LMI_FileSystemConfigurationCapabilities provider implementation.
    """

    INSTANCE_ID = "LMI:LMI_FileSystemConfigurationCapabilities:instance"
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_FileSystemConfigurationCapabilities, self).__init__(
                "LMI_FileSystemConfigurationCapabilities", *args, **kwargs)

        methods = self.Values.SupportedAsynchronousMethods
        fs = self.Values.SupportedActualFileSystemTypes
        availability = self.Values.InitialAvailability
        self.instances = [
            {
                    'InstanceID': self.INSTANCE_ID,
                    'SupportedActualFileSystemTypes': [
                            fs.EXT2,
                            fs.EXT3,
                            fs.EXT4,
                            fs.XFS,
                            fs.BTRFS ],
                    'SupportedSynchronousMethods': pywbem.CIMProperty(
                            name='SupportedSynchronousMethods',
                            value=None,
                            type='uint16',
                            array_size=0,
                            is_array=True),
                    'SupportedAsynchronousMethods':
                            [methods.LMI_CreateFileSystem],
                    'InitialAvailability':  availability.Unmounted
            },
    ]


    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """

        return self.instances

    class Values(CapabilitiesProvider.Values):
        class SupportedActualFileSystemTypes(object):
            Unknown = pywbem.Uint16(0)
            UFS = pywbem.Uint16(2)
            HFS = pywbem.Uint16(3)
            FAT = pywbem.Uint16(4)
            FAT16 = pywbem.Uint16(5)
            FAT32 = pywbem.Uint16(6)
            NTFS4 = pywbem.Uint16(7)
            NTFS5 = pywbem.Uint16(8)
            XFS = pywbem.Uint16(9)
            AFS = pywbem.Uint16(10)
            EXT2 = pywbem.Uint16(11)
            EXT3 = pywbem.Uint16(12)
            REISERFS = pywbem.Uint16(13)
            EXT4 = pywbem.Uint16(0x8001)
            BTRFS = pywbem.Uint16(0x8002)
            JFS = pywbem.Uint16(0x8003)
            TMPFS = pywbem.Uint16(0x8004)
            VFAT = pywbem.Uint16(0x8005)

        class SupportedAsynchronousMethods(object):
            CreateFileSystem = pywbem.Uint16(2)
            DeleteFileSystem = pywbem.Uint16(3)
            ModifyFileSystem = pywbem.Uint16(4)
            CreateGoal = pywbem.Uint16(5)
            GetRequiredStorageSize = pywbem.Uint16(6)
            # DMTF_Reserved = ..
            LMI_CreateFileSystem = pywbem.Uint16(0x8001)

        class SupportedSynchronousMethods(object):
            CreateFileSystem = pywbem.Uint16(2)
            DeleteFileSystem = pywbem.Uint16(3)
            ModifyFileSystem = pywbem.Uint16(4)
            CreateGoal = pywbem.Uint16(5)
            GetRequiredStorageSize = pywbem.Uint16(6)
            # DMTF_Reserved = ..
            LMI_CreateFileSystem = pywbem.Uint16(0x8001)

        class InitialAvailability(object):
            Unknown = pywbem.Uint16(0)
            Mounted = pywbem.Uint16(2)
            Unmounted = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Defined = 0x8000..



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem

class TestCreatePartition(StorageTestBase):
    """
        Test LMI_DiskPartitionConfigurationService.LMI_CreateOrModifyPartition
        with different parameters.
        
        As consequence, it tests also:
        - LMI_DiskPartitionConfigurationCapabilities.CreateSetting.
        - Setting modify / delete.
        - DeleteInstance on partitions.
        - SetPartitionStyle.
    """

    MBR_CLASS = "LMI_DiskPartition"
    GPT_CLASS = "LMI_GenericDiskPartition"

    STYLE_EMBR = 4100
    STYLE_MBR = 2
    STYLE_GPT = 3

    def setUp(self):
        """ Prepare for test."""
        super(TestCreatePartition, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]
        self.storage_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]

    def _check_name(self, partname, classname):
        """ Check that CIMInstanceName represents a partition. """
        self.assertEqual(partname['SystemName'], self.SYSTEM_NAME)
        self.assertEqual(partname['SystemCreationClassName'], self.SYSTEM_CLASS_NAME)
        self.assertEqual(partname['CreationClassName'], classname)

    def _get_capabilities_name(self, style):
        """
            Return CIMInstanceName with partition capabilities representing
            given partition style.
        """
        if style == self.STYLE_EMBR:
            style_name = "EMBR"
        elif style == self.STYLE_MBR:
            style_name = "MBR"
        elif style == self.STYLE_GPT:
            style_name = "GPT"

        return pywbem.CIMInstanceName(
                classname="LMI_DiskPartitionConfigurationCapabilities",
                keybindings={
                        'InstanceID': "LMI:LMI_DiskPartitionConfigurationCapabilities:" + style_name
                })

    def _create_setting(self, style):
        """
            Create new partition configuration setting instance of given
            PartitionStyle and return its CIMInstance.
        """
        capabilities = self._get_capabilities_name(style)
        if not capabilities:
            return None
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "CreateSetting", capabilities)
        self.assertEqual(retval, 0)
        self.assertEqual(len(outparams), 1)
        return self.wbemconnection.GetInstance(outparams['setting'])

    def _delete_setting(self, setting):
        """ Delete Setting with given CIMInstanceName."""
        self.wbemconnection.DeleteInstance(setting)


    def _set_partition_style(self, disk_name, style):
        """ Create partition table on self.disk_name with given style."""
        caps = self._get_capabilities_name(style)
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "SetPartitionStyle",
                self.service,
                Extent=disk_name,
                PartitionStyle=caps)
        self.assertEqual(retval, 0)
        self.assertNocaseDictEqual(outparams, {})

    def _find_extended_partition(self, diskname):
        """
            Return CIMInstanceName of extended partition on the device.
        """
        partitions = self.wbemconnection.Associators(
                diskname,
                AssocClass="LMI_PartitionBasedOn")
        for p in partitions:
            if p['PartitionType'] == 2:  # extended
                return p.path
        return None


    def test_err_no_params(self):
        """ Try LMI_CreateOrModifyPartition with no parameters -> error."""
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "LMI_CreateOrModifyPartition",
                self.service)

    def test_err_wrong_goal(self):
        """ Try LMI_CreateOrModifyPartition with unknown goal InstanceId -> error."""
        goal = pywbem.CIMInstanceName(
                classname="LMI_DiskPartitionConfigurationSetting",
                keybindings={'InstanceID':'not_existing'})
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "LMI_CreateOrModifyPartition",
                self.service,
                extent=self.disk_name,
                Goal=goal)

    def test_err_wrong_extent(self):
        """ Try LMI_CreateOrModifyPartition with unknown extent DeviceID -> error."""
        disk_name = pywbem.CIMInstanceName(
                classname=self.DISK_CLASS,
                keybindings={
                        'DeviceID': '/dev/does/not/exist',
                        'SystemCreationClassName': self.SYSTEM_CLASS_NAME,
                        'SystemName': self.SYSTEM_NAME,
                        'CreationClassName': self.DISK_CLASS})
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "LMI_CreateOrModifyPartition",
                self.service,
                extent=disk_name)

    def test_err_wrong_goal_style_gpt(self):
        """
            Try LMI_CreateOrModifyPartition with goal not corresponding to
            GPT partition table style.-> error.
        """
        # create GPT partition table
        self._set_partition_style(self.disk_name, self.STYLE_GPT)

        # try Hidden
        goal = self._create_setting(self.STYLE_GPT)
        goal['Hidden'] = True
        self.wbemconnection.ModifyInstance(goal)
        self.assertRaises(pywbem.CIMError, self.invoke_async_method,
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=self.disk_name,
                Goal=goal.path)
        self._delete_setting(goal.path)

        # try Extended partition
        goal = self._create_setting(self.STYLE_GPT)
        goal['PartitionType'] = pywbem.Uint16(2)
        self.wbemconnection.ModifyInstance(goal)
        self.assertRaises(pywbem.CIMError, self.invoke_async_method,
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=self.disk_name,
                Goal=goal.path)

        # try logical partition
        goal['PartitionType'] = pywbem.Uint16(3)
        self.wbemconnection.ModifyInstance(goal)
        self.assertRaises(pywbem.CIMError, self.invoke_async_method,
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=self.disk_name,
                Goal=goal.path)

        self._delete_setting(goal.path)

    def test_gpt_maximum(self):
        """
            Try LMI_CreateOrModifyPartition on GPT partition table without
            any size on a disk.
        """
        self._test_gpt_maximum(self.disk_name)


    # TODO: don't skip when #927219 is fixed                )
    @unittest.skip("Cannot create partitions on RAID because of bug #927219")
    def test_gpt_maximum_mdraid(self):
        """
            Try LMI_CreateOrModifyPartition on GPT partition table without
            any size on a MD RAID.
        """
        raidname = self._create_mdraid(self.partition_names[:3], 0)
        self._test_gpt_maximum(raidname)
        self._delete_mdraid(raidname)

    def _test_gpt_maximum(self, disk_name):
        """
            Try LMI_CreateOrModifyPartition on GPT partition table without
            any size on given device.
        """
        # create GPT partition table
        self._set_partition_style(disk_name, self.STYLE_GPT)
        goal = self._create_setting(self.STYLE_GPT)
        (retval, outparams) = self.invoke_async_method(
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=disk_name,
                # TODO: uncomment when #891861 is fixed
                # Goal=goal.path,
                )
        self.assertEqual(retval, 0)
        self.assertIn("partition", outparams)

        partition = outparams['partition']
        disk_instance = self.wbemconnection.GetInstance(disk_name)
        partition_instance = self.wbemconnection.GetInstance(partition)
        self.assertEqual(partition_instance['Primordial'], False)

        self.assertAlmostEqual(
                disk_instance['NumberOfBlocks'],
                partition_instance['NumberOfBlocks'],
                delta=2 * 1024 * 10)  # 10 megabytes

        # second call must fail
        self.assertRaises(pywbem.CIMError, self.invoke_async_method,
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=disk_name,
                # TODO: uncomment when #891861 is fixed
                # Goal=goal.path,
                )

        # check based on
        basedons = self.wbemconnection.References(
                partition,
                ResultClass="LMI_PartitionBasedOn")
        self.assertEqual(len(basedons), 1)
        basedon = basedons[0]
        self.assertCIMNameEquals(basedon['Antecedent'], disk_name)
        self.assertCIMNameEquals(basedon['Dependent'], partition)
        self.assertAlmostEqual(
                basedon['StartingAddress'], 0, delta=2 * 1024 * 2)  # 2 megabytes
        self.assertAlmostEqual(
                basedon['EndingAddress'],
                disk_instance['NumberOfBlocks'],
                delta=4 * 1024 * 2)  # 4 megabytes

        self._delete_setting(goal.path)
        (ret, outparams) = self.invoke_async_method(
                "LMI_DeletePartition",
                self.service,
                int, None,
                Partition=partition)
        self.assertEquals(ret, 0)

    def test_mbr_maximum(self):
        """
            Try LMI_CreateOrModifyPartition on MBR partition table without
            any size on a disk.
        """
        self._test_mbr_maximum(self.disk_name)

    # TODO: don't skip when #927219 is fixed                )
    @unittest.skip("Cannot create partitions on RAID because of bug #927219")
    def test_mbr_maximum_mdraid(self):
        """
            Try LMI_CreateOrModifyPartition on MBR partition table without
            any size on a MD RAID.
        """
        raidname = self._create_mdraid(self.partition_names[:3], 0)
        self._test_mbr_maximum(raidname)
        self._delete_mdraid(raidname)

    def _test_mbr_maximum(self, disk_name):
        """
            Try LMI_CreateOrModifyPartition on MBR partition table without
            any size.
        """
        # create MBR partition table
        self._set_partition_style(disk_name, self.STYLE_MBR)
        goal = self._create_setting(self.STYLE_MBR)
        (retval, outparams) = self.invoke_async_method(
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=disk_name,
                # TODO: uncomment when #891861 is fixed
                # Goal=goal.path,
                )
        self.assertEqual(retval, 0)
        self.assertIn("partition", outparams)

        partition = outparams['partition']
        disk_instance = self.wbemconnection.GetInstance(disk_name)
        partition_instance = self.wbemconnection.GetInstance(partition)
        self.assertEqual(partition_instance['Primordial'], False)

        self.assertAlmostEqual(
                disk_instance['NumberOfBlocks'],
                partition_instance['NumberOfBlocks'],
                delta=2 * 1024 * 10)  # 10 megabytes

        # check based on
        basedons = self.wbemconnection.References(
                partition,
                ResultClass="LMI_PartitionBasedOn")
        self.assertEqual(len(basedons), 1)
        basedon = basedons[0]
        self.assertCIMNameEquals(basedon['Antecedent'], disk_name)
        self.assertCIMNameEquals(basedon['Dependent'], partition)
        self.assertAlmostEqual(
                basedon['StartingAddress'], 0, delta=2 * 1024 * 2)  # 2 megabytes
        self.assertAlmostEqual(
                basedon['EndingAddress'],
                disk_instance['NumberOfBlocks'],
                delta=4 * 1024 * 2)  # 4 megabytes

        # second call must fail
        self.assertRaises(pywbem.CIMError, self.invoke_async_method,
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=disk_name,
                # TODO: uncomment when #891861 is fixed
                # Goal=goal.path,
                )

        self._delete_setting(goal.path)
        (ret, outparams) = self.invoke_async_method(
                "LMI_DeletePartition",
                self.service,
                int, None,
                Partition=partition)
        self.assertEquals(ret, 0)

    def test_gpt_sizes(self):
        """
            Try LMI_CreateOrModifyPartition on GPT partition table with
            sizes on a disk.
        """
        self._test_gpt_sizes(self.disk_name)

    def _test_gpt_sizes(self, disk_name):
        """
            Try LMI_CreateOrModifyPartition on GPT partition table with
            sizes on given device.
        """
        partition_size = 10 * 1024 * 1024  # 10 MB
        partition_count = 10
        partitions = []
        # create GPT partition table
        self._set_partition_style(disk_name, self.STYLE_GPT)
        goal = self._create_setting(self.STYLE_GPT)

        for _ in xrange(partition_count):
            (retval, outparams) = self.invoke_async_method(
                    "LMI_CreateOrModifyPartition",
                    self.service,
                    int, 'partition',
                    extent=disk_name,
                    size=pywbem.Uint64(partition_size),
                    # TODO: uncomment when #891861 is fixed
                    # Goal=goal.path,
                    )
            self.assertEqual(retval, 0)
            self.assertIn("partition", outparams)

            partition = outparams['partition']
            partitions.append(partition)

            partition_instance = self.wbemconnection.GetInstance(partition)
            self.assertEqual(
                    partition_size / 512,
                    partition_instance['NumberOfBlocks'])

            # check based on
            basedons = self.wbemconnection.References(
                    partition,
                    ResultClass="LMI_PartitionBasedOn")
            self.assertEqual(len(basedons), 1)
            basedon = basedons[0]
            self.assertCIMNameEquals(basedon['Antecedent'], disk_name)
            self.assertCIMNameEquals(basedon['Dependent'], partition)

        self._delete_setting(goal.path)

        for partition in partitions:
            (ret, outparams) = self.invoke_async_method(
                    "LMI_DeletePartition",
                    self.service,
                    int, None,
                    Partition=partition)
            self.assertEquals(ret, 0)

    def test_mbr_sizes(self):
        """
            Try LMI_CreateOrModifyPartition on MBR partition table with
            sizes on a disk.
        """
        self._test_mbr_sizes(self.disk_name)

    def _test_mbr_sizes(self, disk_name):
        """
            Try CreateOrModifyPartition on MBR partition table with
            sizes. It should automatically create an extended partition!
        """
        partition_size = 1024 * 1024 * 10  # 10 MB
        partition_count = 10
        partitions = []
        # create MBR partition table
        self._set_partition_style(disk_name, self.STYLE_MBR)

        for i in xrange(partition_count):
            (retval, outparams) = self.invoke_async_method(
                    "LMI_CreateOrModifyPartition",
                    self.service,
                    int, 'partition',
                    extent=disk_name,
                    size=pywbem.Uint64(partition_size),
                    )
            self.assertEqual(retval, 0)
            self.assertIn("partition", outparams)

            partition = outparams['partition']
            partitions.append(partition)

            partition_instance = self.wbemconnection.GetInstance(partition)
            self.assertEqual(
                    partition_size / 512,
                    partition_instance['NumberOfBlocks'])

            # check based on
            basedons = self.wbemconnection.References(
                    partition,
                    ResultClass="LMI_PartitionBasedOn")
            self.assertEqual(len(basedons), 1)
            basedon = basedons[0]
            if i < 3:
                self.assertCIMNameEquals(basedon['Antecedent'], disk_name)
            else:
                if i == 3:
                    # find the extended partition
                    extended_partition = self._find_extended_partition(disk_name)
                # it's based on the extended partition!
                self.assertNotEqual(basedon['Antecedent'], extended_partition)
            self.assertCIMNameEquals(basedon['Dependent'], partition)

        partitions.reverse()
        for partition in partitions:
            (ret, outparams) = self.invoke_async_method(
                    "LMI_DeletePartition",
                    self.service,
                    int, None,
                    Partition=partition)
            self.assertEquals(ret, 0)
        (ret, outparams) = self.invoke_async_method(
                "LMI_DeletePartition",
                self.service,
                int, None,
                Partition=extended_partition)
        self.assertEquals(ret, 0)

        # remove the extended partition, created automatically

    # TODO: don't skip when #891861 is fixed                )
    @unittest.skip("Cannot create extended partitions because of bug #891861")
    def test_logical_sizes(self):
        """
            Try LMI_CreateOrModifyPartition on MBR partition table with
            start/end addresses and logical partitions.
        """
        partition_size = 1024 * 1024 * 10  # 10 MB
        partition_count = 10
        partitions = []

        # create MBR partition table and one huge Extended partition
        self._set_partition_style(self.disk_name, self.STYLE_MBR)
        goal = self._create_setting(self.STYLE_MBR)
        goal['PartitionType'] = pywbem.Uint16(2)  # Extended
        self.wbemconnection.ModifyInstance(goal)
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "LMI_CreateOrModifyPartition",
                self.service,
                extent=self.disk_name,
                Goal=goal.path)
        self.assertEqual(retval, 0)
        self.assertIn("partition", outparams)
        extended_partition = outparams['partition']

        # create partition_count logical partitions on it
        goal['PartitionType'] = pywbem.Uint16(3)  # Logical
        self.wbemconnection.ModifyInstance(goal)
        for _ in xrange(partition_count):
            (retval, outparams) = self.wbemconnection.InvokeMethod(
                    "LMI_CreateOrModifyPartition",
                    self.service,
                    extent=extended_partition,
                    Size=pywbem.Uint64(partition_size),
                    Goal=goal.path)
            self.assertEqual(retval, 0)
            self.assertIn("partition", outparams)

            partition = outparams['partition']
            partitions.append(partition)

            partition_instance = self.wbemconnection.GetInstance(partition)
            self.assertEqual(
                    partition_size / 512,
                    partition_instance['NumberOfBlocks'])

            # check based on
            basedons = self.wbemconnection.References(
                    partition,
                    ResultClass="LMI_PartitionBasedOn")
            self.assertEqual(len(basedons), 1)
            basedon = basedons[0]
            self.assertNotEqual(basedon['Antecedent'], extended_partition)
            self.assertEqual(basedon['Dependent'], partition)

        self._delete_setting(goal.path)

        partitions.reverse()
        for partition in partitions:
            (ret, outparams) = self.invoke_async_method(
                    "LMI_DeletePartition",
                    self.service,
                    int, None,
                    Partition=partition)
            self.assertEquals(ret, 0)
        (ret, outparams) = self.invoke_async_method(
                "LMI_DeletePartition",
                self.service,
                int, None,
                Partition=extended_partition)
        self.assertEquals(ret, 0)

    # TODO: test partition modification

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_SystemStorageDevice class.

LMI_SystemStorageDevice
-----------------------

.. autoclass:: LMI_SystemStorageDevice
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
import blivet.devices
from lmi.providers.ComputerSystem import get_system_name

LOG = cmpi_logging.get_logger(__name__)

class LMI_SystemStorageDevice(BaseProvider):
    """
        Implementation of LMI_SystemStorageDevice provider.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_SystemStorageDevice, self).__init__(*args, **kwargs)


    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        # just check keys
        system = model['GroupComponent']
        if (system['CreationClassName'] != self.config.system_class_name
                or system['Name'] != get_system_name()):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Wrong GroupComponent keys.")

        device_name = model['PartComponent']
        device = self.provider_manager.get_device_for_name(device_name)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Wrong PartComponent keys.")

        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'GroupComponent': None, 'PartComponent': None})
        for device in self.storage.devices:
            if isinstance(device, blivet.devices.LVMVolumeGroupDevice):
                continue  # Pools are not LogicalDevices

            model['GroupComponent'] = pywbem.CIMInstanceName(
                    classname=self.config.system_class_name,
                    namespace=self.config.namespace,
                    keybindings={
                            'CreationClassName' : self.config.system_class_name,
                            'Name' : get_system_name(),
                    })
            provider = self.provider_manager.get_provider_for_device(device)
            if not provider:
                LOG().trace_warn("Cannot find provider for %s", device.path)
                continue
            model['PartComponent'] = provider.get_name_for_device(device)
            yield model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_System",
                "CIM_StorageExtent")



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem
import subprocess
import tempfile
import os

EXT2 = 11  # LMI_FileSystemSetting.ActualFileSystemType

class TestOverwriteUnused(StorageTestBase):
    """
        Test that 'unused' devices can be overwritten in various ways.
    """

    def call(self, *args, **kwargs):
        return subprocess.call(*args, close_fds=True, **kwargs)

    def check_call(self, *args, **kwargs):
        return subprocess.check_call(*args, close_fds=True, **kwargs)

    def setUp(self):
        super(TestOverwriteUnused, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_FileSystemConfigurationService")[0]
        self.storage_service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_StorageConfigurationService")[0]
        # create a directory to mount stuff
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        os.rmdir(self.tmpdir)


    def _prepare_nonfs_partitions(self):
        """
        Prepare some metadata on partitions.
        :returns: tuple (array of integers, array of integers).
        Both arrays are indexes into self.partition_names, indicating
        used partitions (=the first array) and unused
        partitions(=the second array).
        The used partitions are not allowed to be deleted or overwritten.
        """
        # partition 0+1: stopped RAID = unused
        self.check_call(['mdadm', '-C', '-l', '0', '-n', '2',
                '/dev/md/second', self.partitions[0], self.partitions[1]])
        self.check_call(['mdadm', '-S', '/dev/md/second'])

        # partition 2+3: running RAID = used
        self.check_call(['mdadm', '-C', '-l', '0', '-n', '2',
                '/dev/md/first', self.partitions[2], self.partitions[3]])

        # partition 4: unused PV = unused
        self.check_call(['pvcreate', self.partitions[4]])
        # partition 5: used PV = used
        self.check_call(['pvcreate', self.partitions[5]])
        self.check_call(['vgcreate', 'tstVG', self.partitions[5]])

        # workaround for bug #947815
        self.check_call(['udevadm','trigger'])

        # TODO: add LUKS
        self.restart_cim()
        return ([2, 3, 5], [0, 1, 4])


    def _clear_nonfs_partitions(self):
        """
        Clear all data created by _prepare_nonfs.
        """
        self.call(['mdadm', '-S', '/dev/md/first'])
        self.call(['pvremove', self.partitions[4]])
        self.call(['vgremove', 'tstVG'])
        self.call(['pvremove', self.partitions[5]])
        for part in self.partitions:
            self.call(['wipefs', '-a', '--force', part])

        self.restart_cim()

    def _prepare_fs_partitions(self, for_delete):
        """
        Prepare some filesystems on partitions.
        :param for_delete: ``boolean`` True, if the filesystems will be deleted.
        I.e. btrfs on two devices deletes formats on both devices, while
        formatting one device as different format leaves the other formatted
        as btrfs.

        :returns: tuple (array of integers, array of integers).
        Both arrays are indexes into self.partition_names, indicating
        used partitions (=the first array) and unused
        partitions(=the second array).
        The used partitions are not allowed to be deleted or overwritten.
        """
        # create a directory to mount stuff
        os.mkdir(self.tmpdir + '/1')
        os.mkdir(self.tmpdir + '/2')

        # partition 0+1: unmounted BTRFS = unused
        # This does not work because of bug #948302
        # TODO: uncomment when 948302 is fixed
        # self.check_call(['mkfs.btrfs', '-f',
        #        self.partitions[0], self.partitions[1]])

        # partition 2+3: mounted BTRFS = used
        self.check_call(['mkfs.btrfs', '-f',
                self.partitions[2], self.partitions[3]])
        self.check_call(['mount', self.partitions[2], self.tmpdir + '/1'])

        # partition 4: unmounted ext3 = unused
        self.check_call(['mkfs.ext3', self.partitions[4]])
        # partition 5: mounted ext3 = used
        self.check_call(['mkfs.ext3', self.partitions[5]])
        self.check_call(['mount', self.partitions[5], self.tmpdir + '/2'])
        # partition 6: unmounted btrfs = unused
        self.check_call(['mkfs.btrfs', '-f', self.partitions[6]])

        self.restart_cim()

        if for_delete:
            # partition[1] was erased by removing format from partitions[0],
            # there is no format on it
            return ([2, 3, 5], [4, 6])
        else:
            return ([2, 3, 5], [4, 6])


    def _clear_fs_partitions(self):
        """
        Clear all data created by _prepare_fs.
        """
        self.call(['umount', self.tmpdir + '/2'])
        self.call(['umount', self.tmpdir + '/1'])
        for part in self.partitions:
            self.call(['wipefs', '-a', '--force', part])
        os.rmdir(self.tmpdir + '/2')
        os.rmdir(self.tmpdir + '/1')
        self.restart_cim()

    def _test_create_fs(self, used, unused):
        """ Test LMI_CreateFileSystem both on used and unused devices."""
        for i in unused + used:
            device_name = self.partition_names[i]

            try:
                # Try to reformat the partition and see what happens
                (ret, _outparams) = self.invoke_async_method(
                        "LMI_CreateFileSystem",
                        self.service,
                        int, None,
                        InExtents=[device_name, ],
                        FileSystemType=pywbem.Uint16(EXT2))
            except pywbem.CIMError, err:
                if i not in unused:
                    pass  # we expected the error
                else:
                    print "Got unexpected exception:", err
                ret = 1

            if i in unused:
                # LMI_CreateFileSystem must have succeeded
                self.assertEquals(ret, 0)
            else:
                # LMI_CreateFileSystem must have failed
                self.assertNotEquals(ret, 0)


    def _test_delete_fs(self, used, unused):
        """ Test DeleteFileSystem both on used and unused devices."""
        for i in unused + used:
            device_name = self.partition_names[i]
            # find associated format
            formats = self.wbemconnection.AssociatorNames(device_name,
                    AssocClass='LMI_ResidesOnExtent')
            self.assertEquals(len(formats), 1)
            fmt = formats[0]
            try:
                # Try to delete the format
                (ret, _outparams) = self.invoke_async_method(
                        "DeleteFileSystem",
                        self.service,
                        int, None,
                        TheFileSystem=fmt)
            except pywbem.CIMError, err:
                if i not in unused:
                    pass  # we expected the error
                else:
                    print "Got unexpected exception:", err
                ret = 1

            if i in unused:
                # DeleteFileSystem must have succeeded
                self.assertEquals(ret, 0)
            else:
                # DeleteFileSystem must have failed
                self.assertNotEquals(ret, 0)

    def _test_create_vg(self, used, unused):
        """ Test CreateOrModifyVG both on used and unused devices."""
        for i in unused + used:
            device_name = self.partition_names[i]
            try:
                # Try to create a VG on the device
                (ret, outparams) = self.invoke_async_method(
                        "CreateOrModifyVG",
                        self.storage_service,
                        int, 'pool',
                        InExtents=[device_name, ],
                        ElementName='tst')
            except pywbem.CIMError, err:
                if i not in unused:
                    pass  # we expected the error
                else:
                    print "Got unexpected exception:", err

                ret = 1

            if i in unused:
                # CreateOrModifyVG must have succeeded
                self.assertEquals(ret, 0)
                # delete the VG
                poolname = outparams['pool']
                (ret, _outparams) = self.invoke_async_method(
                        "DeleteVG",
                        self.storage_service,
                        int, None,
                        Pool=poolname)
                self.assertEquals(ret, 0)
            else:
                # CreateOrModifyVG must have failed
                self.assertNotEquals(ret, 0)

    def _test_create_raid(self, used, unused):
        """ Test CreateOrModifyMDRAID both on used and unused devices."""
        for i in unused + used:
            # We need at least two devices for MD RAID, take some unused
            # as the second and make sure it's not the tested one.
            second = unused[0]
            if second == i:
                second = unused[1]

            device_name = self.partition_names[i]
            second_name = self.partition_names[second]
            try:
                # Try to create a RAID on the device
                (ret, outparams) = self.invoke_async_method(
                        "CreateOrModifyMDRAID",
                        self.storage_service,
                        int, 'theelement',
                        InExtents=[second_name, device_name, ],
                        Level=pywbem.Uint16(0),
                        ElementName='tst')
            except pywbem.CIMError, err:
                if i not in unused:
                    pass  # we expected the error
                else:
                    print "Got unexpected exception:", err

                ret = 1

            if i in unused:
                # CreateOrModifyMDRAID must have succeeded
                self.assertEquals(ret, 0)
                # delete the raid
                raidname = outparams['theelement']
                (ret, _outparams) = self.invoke_async_method(
                        "DeleteMDRAID",
                        self.storage_service,
                        int, None,
                        TheElement=raidname)
                self.assertEquals(ret, 0)
            else:
                # CreateOrModifyMDRAID must have failed
                self.assertNotEquals(ret, 0)

    def _test_nonfs(self, test_function):
        """
        Prepare various non-fs formats on devices and call test_function on
        them.
        """
        try:
            (used, unused) = self._prepare_nonfs_partitions()
            test_function(used, unused)
        finally:
            self._clear_nonfs_partitions()

    def _test_fs(self, test_function, for_delete):
        """
        Prepare various fs formats on devices and call test_function on
        them.
        """
        try:
            (used, unused) = self._prepare_fs_partitions(for_delete)
            test_function(used, unused)
        finally:
            self._clear_fs_partitions()

    def test_create_fs_nonfs(self):
        """Test LMI_CreateFileSystem on non-fs formats."""
        self._test_nonfs(self._test_create_fs)

    def test_delete_fs_nonfs(self):
        """Test DeleteFileSystem on non-fs formats."""
        self._test_nonfs(self._test_delete_fs)

    def test_create_vg_nonfs(self):
        """Test CreateOrModifyVG on non-fs formats."""
        self._test_nonfs(self._test_create_vg)

    def test_create_raid_nonfs(self):
        """Test CreateOrModifyMDRAID on non-fs formats."""
        self._test_nonfs(self._test_create_raid)

    def test_create_fs_fs(self):
        """Test LMI_CreateFileSystem on fs formats."""
        self._test_fs(self._test_create_fs, False)

    def test_delete_fs_fs(self):
        """Test DeleteFileSystem on fs formats."""
        self._test_fs(self._test_delete_fs, True)

    def test_create_vg_fs(self):
        """Test CreateOrModifyVG on fs formats."""
        self._test_fs(self._test_create_vg, False)

    def test_create_raid_fs(self):
        """Test CreateOrModifyMDRAID on fs formats."""
        self._test_fs(self._test_create_raid, False)

    def test_delete_raid(self):
        """ Test DeleteMDRAID on used and unused MD RAID device."""
        try:
            os.mkdir(self.tmpdir + "/second")

            # first: unused
            self.check_call(['mdadm', '-C', '-l', '0', '-n', '2',
                    '/dev/md/first', self.partitions[0], self.partitions[1]])

            # second: with mounted fs = used
            self.check_call(['mdadm', '-C', '-l', '0', '-n', '2',
                    '/dev/md/second', self.partitions[2], self.partitions[3]])
            self.check_call(['mkfs.ext3', '/dev/md/second'])
            self.check_call(['mount', '/dev/md/second',
                    self.tmpdir + "/second"])

            # third: part of VG = used
            self.check_call(['mdadm', '-C', '-l', '0', '-n', '2',
                    '/dev/md/third', self.partitions[4], self.partitions[5]])
            self.check_call(['pvcreate', '/dev/md/third'])
            self.check_call(['vgcreate', 'thirdVG', '/dev/md/third'])
            # workaround for bug #947815
            self.check_call(['udevadm','trigger'])
            self.restart_cim()

            used = ['/dev/md/third', '/dev/md/second']
            unused = ['/dev/md/first']

            for devname in unused + used:
                device = self.wbemconnection.ExecQuery("WQL",
                        'SELECT * FROM LMI_StorageExtent WHERE Name="%s"'
                        % (devname,))[0]
                try:
                    # Try to create a VG on the device
                    (ret, _outparams) = self.invoke_async_method(
                            "DeleteMDRAID",
                            self.storage_service,
                            int, None,
                            TheElement=device.path)
                except pywbem.CIMError, err:
                    if devname not in unused:
                        pass  # we expected the error
                    else:
                        print "Got unexpected exception:", err
                    ret = 1

                if devname in unused:
                    # DeleteMDRAID must have succeeded
                    self.assertEquals(ret, 0)
                else:
                    # DeleteMDRAID must have failed
                    self.assertNotEquals(ret, 0)
        finally:
            self.call(['vgremove', 'thirdVG'])
            self.call(['umount', '/dev/md/second'])
            self.call(['mdadm', '-S', '/dev/md/first'])
            self.call(['mdadm', '-S', '/dev/md/second'])
            self.call(['mdadm', '-S', '/dev/md/third'])
            os.rmdir(self.tmpdir + "/second")
            for part in self.partitions:
                self.call(['wipefs', '-a', '--force', part])
            self.restart_cim()


    def test_delete_vg(self):
        """ Test DeleteVG on used and unused MD RAID device."""
        try:
            os.mkdir(self.tmpdir + "/second")

            for part in self.partitions:
                self.check_call(['pvcreate', part])

            # VGfirst: unused
            self.check_call(['vgcreate', 'VGfirst',
                    self.partitions[0], self.partitions[1]])

            # VGsecond: mounted = used
            self.check_call(['vgcreate', 'VGsecond',
                    self.partitions[2], self.partitions[3]])
            self.check_call(['lvcreate', '-n', 'LVsecond', '-l',
                    '100%VG', 'VGsecond'])
            self.check_call(['mkfs.ext3', '/dev/VGsecond/LVsecond'])
            self.check_call(['mount', '/dev/VGsecond/LVsecond',
                    self.tmpdir + "/second"])

            # VGthird: has LV = used
            self.check_call(['vgcreate', 'VGthird',
                    self.partitions[4], self.partitions[5]])
            self.check_call(['lvcreate', '-n', 'LVthird', '-l',
                    '100%VG', 'VGthird'])
            self.restart_cim()

            used = ['VGsecond', 'VGthird']
            unused = ['VGfirst']

            for poolname in unused + used:
                pool = self.wbemconnection.ExecQuery("WQL",
                        'SELECT * FROM LMI_VGStoragePool WHERE ElementName="%s"'
                        % (poolname,))[0]
                try:
                    # Try to create a VG on the device
                    (ret, _outparams) = self.invoke_async_method(
                            "DeleteVG",
                            self.storage_service,
                            int, None,
                            Pool=pool.path)
                except pywbem.CIMError, err:
                    if poolname not in unused:
                        pass  # we expected the error
                    else:
                        print "Got unexpected exception:", err
                    ret = 1

                if poolname in unused:
                    # DeleteVG must have succeeded
                    self.assertEquals(ret, 0)
                else:
                    # DeleteVG must have failed
                    self.assertNotEquals(ret, 0)
        finally:
            self.call(['lvremove', '-f', '/dev/VGthird/LVthird'])
            self.call(['vgremove', 'VGthird'])
            self.call(['umount', '/dev/VGsecond/LVsecond'])
            self.call(['lvremove', '-f', '/dev/VGsecond/LVsecond'])
            self.call(['vgremove', 'VGsecond'])
            self.call(['vgremove', 'VGfirst'])
            os.rmdir(self.tmpdir + "/second")
            for part in self.partitions:
                self.call(['wipefs', '-a', '--force', part])
            self.restart_cim()

    def test_delete_lv(self):
        """ Test LV on used and unused MD RAID device."""
        try:
            os.mkdir(self.tmpdir + "/second")

            for part in self.partitions:
                self.check_call(['pvcreate', part])

            self.check_call(['vgcreate', 'VG',
                    self.partitions[0], self.partitions[1]])

            # LVfirst - unused
            self.check_call(['lvcreate', '-n', 'LVfirst', '-l',
                    '50%VG', 'VG'])

            # LVsecond - mounted = used
            self.check_call(['lvcreate', '-n', 'LVsecond', '-l',
                    '50%VG', 'VG'])
            self.check_call(['mkfs.ext3', '/dev/VG/LVsecond'])
            self.check_call(['mount', '/dev/VG/LVsecond',
                    self.tmpdir + "/second"])
            self.restart_cim()

            used = ['LVsecond']
            unused = ['LVfirst']

            for devname in unused + used:
                device = self.wbemconnection.ExecQuery("WQL",
                        'SELECT * FROM LMI_LVStorageExtent WHERE ElementName="%s"'
                        % (devname,))[0]
                try:
                    # Try to create a VG on the device
                    (ret, _outparams) = self.invoke_async_method(
                            "DeleteLV",
                            self.storage_service,
                            int, None,
                            TheElement=device.path)
                except pywbem.CIMError, err:
                    if devname not in unused:
                        pass  # we expected the error
                    else:
                        print "Got unexpected exception:", err
                    ret = 1

                if devname in unused:
                    # DeleteLV must have succeeded
                    self.assertEquals(ret, 0)
                else:
                    # DeleteLV must have failed
                    self.assertNotEquals(ret, 0)
        finally:
            self.call(['umount', '/dev/VG/LVsecond'])
            self.call(['lvremove', '-f', '/dev/VG/LVsecond'])
            self.call(['lvremove', '-f', '/dev/VG/LVfirst'])
            self.call(['vgremove', 'VG'])
            os.rmdir(self.tmpdir + "/second")
            for part in self.partitions:
                self.call(['wipefs', '-a', '--force', part])
            self.restart_cim()

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>
# -*- coding: utf-8 -*-
""" Module for MountingProvider class. """

import pywbem
import blivet
from lmi.storage.BaseProvider import BaseProvider
import lmi.providers.cmpi_logging as cmpi_logging

class MountingProvider(BaseProvider):
    """
    An auxiliary class that provides some code sharing and a common base for all
    mounting classes.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        """
           init
        """
        super(MountingProvider, self).__init__(*args, **kwargs)

    def get_device_and_format_from_fs(self, fs):
        provider = self.provider_manager.get_provider_for_format_name(fs)

        if provider is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "Could not get provider for %s" % fs['Name'])

        (device, fmt) = provider.get_format_for_name(fs)

        if device is None:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "No such device: " + fs['Name'])

        return (device, fmt)

    def check_get_instance(self, device, spec, path):
        if isinstance(device, blivet.devices.NoDevice):
            paths = [device.format.mountpoint]
        else:
            paths = blivet.util.get_mount_paths(device.path)

        if device is None or not paths:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "No such mounted device: " + spec)
        if path not in paths and device.path != spec:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED, "%s is not mounted here: %s" % (spec, path))



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for FormatProvider.

LMI_DataFormatProvider
----------------------

.. autoclass:: LMI_DataFormatProvider
    :members:

"""

from lmi.storage.FormatProvider import FormatProvider
import blivet.formats
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_DataFormatProvider(FormatProvider):
    """
        Provider of generic LMI_DataFormat.
        It provides all DeviceFormats, which do not have any special provider
        (like MDRAIDFormat or PVFormat) and are not filesystems.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_DataFormatProvider, self).__init__(
                "LMI_DataFormat",
                None,
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_format(self, device, fmt):
        if fmt is None:
            return False
        if isinstance(fmt, blivet.formats.mdraid.MDRaidMember):
            return False
        if isinstance(fmt, blivet.formats.lvmpv.LVMPhysicalVolume):
            return False
        if isinstance(fmt, blivet.formats.fs.FS):
            return False

        # skip 'Unknown' format
        if fmt.type is None:
            return False
        # skip partition tables
        if isinstance(fmt, blivet.formats.disklabel.DiskLabel):
            return False

        return True



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Synacek <jsynacek@redhat.com>

import blivet.devices
from lmi.storage.ExtentProvider import ExtentProvider
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_LUKSStorageExtent(ExtentProvider):
    """
        Provider of LMI_LUKSStorageExtent class.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LUKSStorageExtent, self).__init__(
            classname='LMI_LUKSStorageExtent',
            *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_device(self, device):
        """
            Returns True, if this class is provider for given Anaconda
            StorageDevice class.
        """
        if  isinstance(device, blivet.devices.LUKSDevice):
            return device.status
        return False

    @cmpi_logging.trace_method
    def enumerate_devices(self):
        """
            Enumerate all StorageDevices, that this provider provides.
        """
        for device in self.storage.devicetree.getDevicesByInstance(blivet.devices.LUKSDevice):
            if device.status:
                yield device




# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_DiskPartitionConfigurationCapabilities class.

LMI_DiskPartitionConfigurationCapabilities
------------------------------------------

.. autoclass:: LMI_DiskPartitionConfigurationCapabilities
    :members:

"""

from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
from lmi.storage.LMI_DiskPartitionConfigurationSetting \
        import LMI_DiskPartitionConfigurationSetting
from lmi.storage.SettingManager import Setting
import pywbem
import blivet.formats
import parted
import lmi.providers.cmpi_logging as cmpi_logging
import lmi.storage.util.units as units

class LMI_DiskPartitionConfigurationCapabilities(CapabilitiesProvider):
    """
        LMI_DiskPartitionConfigurationCapabilities provider implementation.
    """

    INSTANCE_ID_MBR = "LMI:LMI_DiskPartitionConfigurationCapabilities:MBR"
    INSTANCE_ID_EMBR = "LMI:LMI_DiskPartitionConfigurationCapabilities:EMBR"
    INSTANCE_ID_GPT = "LMI:LMI_DiskPartitionConfigurationCapabilities:GPT"

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_DiskPartitionConfigurationCapabilities, self).__init__(
                "LMI_DiskPartitionConfigurationCapabilities", *args, **kwargs)

        sync_actions = self.Values.SupportedSynchronousActions
        self.instances = [
            {
                    'InstanceID': self.INSTANCE_ID_MBR,
                    'SupportedSettings': [
                            self.Values.SupportedSettings.Partition_Type,
                            self.Values.SupportedSettings.Bootable],
                    'PartitionTableSize': pywbem.Uint32(1),
                    'PartitionStyle': self.Values.PartitionStyle.MBR,
                    'ValidSubPartitionStyles': [
                            self.Values.ValidSubPartitionStyles.EMBR],
                    'MaxNumberOfPartitions': pywbem.Uint16(4),
                    'SupportedSynchronousActions': [
                            sync_actions.CreateOrModifyPartition,
                            sync_actions.SetPartitionStyle],
                    'Caption': "Capabilities of MSDOS style primary " \
                                "partitions.",
                    'MaxCapacity': pywbem.Uint64(units.MAXINT64),
                    'ElementName': 'MBRCapabilities',
                    'OverlapAllowed' : False,
                    # TODO: add Hidden flag? It seems it does not work
                    # on MBR partitions
            },
            {
                    'InstanceID': self.INSTANCE_ID_EMBR,
                    'SupportedSettings': [
                            self.Values.SupportedSettings.Bootable],
                    'PartitionTableSize': pywbem.Uint32(1),
                    'PartitionStyle': self.Values.PartitionStyle.EMBR,
                    'ValidSubPartitionStyles': pywbem.CIMProperty(
                            name='ValidSubPartitionStyles',
                            value=None,
                            type='uint16',
                            array_size=0,
                            is_array=True),
                    'MaxNumberOfPartitions': pywbem.Uint16(2 << 15 - 1),
                    'SupportedSynchronousActions': [
                            sync_actions.CreateOrModifyPartition,
                            sync_actions.SetPartitionStyle],
                    'Caption':
                            "Capabilities of MS-DOS style logical partitions.",
                    'MaxCapacity': pywbem.Uint64(units.MAXINT64),
                    'ElementName': 'EMBRCapabilities',
                    'OverlapAllowed' : False,
            },
            {
                    'InstanceID': self.INSTANCE_ID_GPT,
                    'SupportedSettings': [
                            self.Values.SupportedSettings.Bootable, ],
                    'PartitionTableSize': pywbem.Uint32(68),
                    'PartitionStyle': self.Values.PartitionStyle.GPT,
                    'ValidSubPartitionStyles': pywbem.CIMProperty(
                            name='ValidSubPartitionStyles',
                            value=None,
                            type='uint16',
                            array_size=0,
                            is_array=True),
                    'MaxNumberOfPartitions': pywbem.Uint16(128),
                    'SupportedSynchronousActions': [
                            sync_actions.CreateOrModifyPartition,
                            sync_actions.SetPartitionStyle],
                    'Caption': "Capabilities of GPT partitions.",
                    'MaxCapacity': pywbem.Uint64(units.MAXINT64),
                    'ElementName': 'GPTCapabilities',
                    'OverlapAllowed' : False,
                    CapabilitiesProvider.DEFAULT_CAPABILITY: True,
            },
    ]


    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """

        return self.instances

    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        """
            Create LMI_*Setting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.
        """
        setting_id = self.setting_manager.allocate_id(
                'LMI_DiskPartitionConfigurationSetting')
        if not setting_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Failed to allocate setting InstanceID")

        values = LMI_DiskPartitionConfigurationSetting.Values
        setting = self.setting_manager.create_setting(
                'LMI_DiskPartitionConfigurationSetting',
                Setting.TYPE_TRANSIENT,
                setting_id)
        setting['Bootable'] = False
        setting['ElementName'] = 'CreatedFrom' + capabilities['InstanceID']
        setting['Hidden'] = None
        if (capabilities['PartitionStyle'] == self.Values.PartitionStyle.GPT
                or capabilities['PartitionStyle'] ==
                        self.Values.PartitionStyle.MBR):
            setting['PartitionType'] = values.PartitionType.Primary
        else:
            setting['PartitionType'] = values.PartitionType.Logical

        self.setting_manager.set_setting(
                'LMI_DiskPartitionConfigurationSetting', setting)

        return pywbem.CIMInstanceName(
                classname='LMI_DiskPartitionConfigurationSetting',
                namespace=self.config.namespace,
                keybindings={'InstanceID': setting_id})

    @cmpi_logging.trace_method
    def cim_method_getalignment(self, env, object_name,
                                param_extent=None):
        """
        Implements LMI_DiskPartitionConfigurationCapabilities.GetAlignment()

        Return allignment unit for given StorageExtent (in blocks). New
        partitions and metadata sectors should be aligned to this unit.
        """
        capabilities = self.get_capabilities_for_id(object_name['InstanceID'])
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Capabilities not found.")

        if not param_extent:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter Extent must be provided.")
        device = self.provider_manager.get_device_for_name(param_extent)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Extent not found.")
        self.check_capabilities_for_device(device, capabilities)

        if capabilities['PartitionStyle'] == self.Values.PartitionStyle.EMBR:
            device = device.parents[0]

        alignment = device.partedDevice.optimumAlignment.grainSize

        out_params = [pywbem.CIMParameter('alignment', type='uint64',
                value=pywbem.Uint64(alignment))]
        retval = self.Values.GetAlignment.Success
        return (retval, out_params)

    @cmpi_logging.trace_method
    def get_capabilities_for_device(self, device):
        """
            Return capabilities for given StorageDevice
            Return None if it does not have any partition capabilities.
        """

        if isinstance(device, blivet.devices.PartitionDevice):
            if device.isExtended:
                return self.get_capabilities_for_id(self.INSTANCE_ID_EMBR)
            return None

        fmt_class = blivet.formats.disklabel.DiskLabel
        if (not device.format) or (not isinstance(device.format, fmt_class)):
            return None

        if device.format.labelType == "msdos":
            return self.get_capabilities_for_id(self.INSTANCE_ID_MBR)
        if device.format.labelType == "gpt":
            return self.get_capabilities_for_id(self.INSTANCE_ID_GPT)

    @cmpi_logging.trace_method
    def check_capabilities_for_device(self, device, capabilities):
        """
            Check if the capabilities are the right one for the device and
            raise exception if something is wrong.
        """
        if capabilities['PartitionStyle'] == self.Values.PartitionStyle.MBR:
            if (not device.format or not isinstance(
                            device.format,
                            blivet.formats.disklabel.DiskLabel)):
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "There is no partition table on the Extent.")
            if device.format.labelType != "msdos":
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Partition table does not have given Capabilities.")

        elif capabilities['PartitionStyle'] == self.Values.PartitionStyle.GPT:
            if (not device.format or not isinstance(
                            device.format,
                            blivet.formats.disklabel.DiskLabel)):
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "There is no partition table on the Extent.")
            if device.format.labelType != "gpt":
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Partition table does not have given Capabilities.")

        else:
            # the device must be extended partition, find its disk
            if not isinstance(
                    device, blivet.devices.PartitionDevice):
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "The extent is not an partition.")
            if not device.isExtended:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "The extent is not an extended partition.")

        return True

    @cmpi_logging.trace_method
    def cim_method_findpartitionlocation(self, env, object_name,
                                         param_extent=None,
                                         param_size=None):
        """
        Implements LMI_DiskPartitionConfigurationCapabilities
        .FindPartitionLocation()

        This method finds the best place for partition of given size.
        """
        capabilities = self.get_capabilities_for_id(object_name['InstanceID'])
        if not capabilities:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Capabilities not found.")

        # Check parameters
        if not param_extent:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter Extent must be provided.")
        device = self.provider_manager.get_device_for_name(param_extent)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Extent not found.")

        # Check device vs capabilities
        self.check_capabilities_for_device(device, capabilities)

        if capabilities['PartitionStyle'] == self.Values.PartitionStyle.EMBR:
            device = device.parents[0]
            part_type = parted.PARTITION_LOGICAL
        else:
            part_type = parted.PARTITION_NORMAL

        if not param_size:
            grow = True
            size = 1.0
        else:
            grow = False
            size = param_size / (units.MEGABYTE * 1.0)

        # Find the best place
        geometry = blivet.partitioning.getBestFreeSpaceRegion(
                disk=device.format.partedDisk,
                part_type=part_type,
                req_size=size,
                grow=grow)

        if not geometry:
            # No place found
            retval = self.Values.FindPartitionLocation.Not_Enough_Free_Space
            out_params = [pywbem.CIMParameter('size', type='uint64',
                    value=pywbem.Uint64(0))]
            return (retval, out_params)

        retval = self.Values.FindPartitionLocation.Success

        sector_size = device.partedDevice.sectorSize
        new_size = geometry.length * device.partedDevice.sectorSize

        # anaconda returns the whole region size, we should make it smaller
        # to adjust to requested size
        if not grow and new_size > param_size:
            # truncate the end to start + size
            geometry.end = geometry.start + (param_size / sector_size)
            if param_size % sector_size > 0:
                geometry.end = geometry.end + 1
        new_size = (geometry.end - geometry.start) * sector_size

        out_params = [pywbem.CIMParameter('size', type='uint64',
                value=pywbem.Uint64(new_size))]
        out_params += [pywbem.CIMParameter('startingaddress', type='uint64',
                value=pywbem.Uint64(geometry.start))]
        out_params += [pywbem.CIMParameter('endingaddress', type='uint64',
                value=pywbem.Uint64(geometry.end))]
        return (retval, out_params)


    class Values(CapabilitiesProvider.Values):
        class SupportedSettings(object):
            Partition_Type = pywbem.Uint16(1)
            Bootable = pywbem.Uint16(2)
            Hidden = pywbem.Uint16(3)

        class ValidSubPartitionStyles(object):
            Other = pywbem.Uint16(1)
            MBR = pywbem.Uint16(2)
            VTOC = pywbem.Uint16(3)
            GPT = pywbem.Uint16(4)
            EMBR = pywbem.Uint16(4100)

        class SupportedSynchronousActions(object):
            SetPartitionStyle = pywbem.Uint16(2)
            CreateOrModifyPartition = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Reserved = 0x8000..

        class PartitionStyle(object):
            MBR = pywbem.Uint16(2)
            GPT = pywbem.Uint16(3)
            VTOC = pywbem.Uint16(4)
            PC98 = pywbem.Uint16(4097)
            SUN = pywbem.Uint16(4098)
            MAC = pywbem.Uint16(4099)
            EMBR = pywbem.Uint16(4100)

        class GetAlignment(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Failed = pywbem.Uint32(4)

        class FindPartitionLocation(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Failed = pywbem.Uint32(4)
            Not_Enough_Free_Space = pywbem.Uint32(100)



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LVAllocatedFromStoragePool class.

LMI_LVAllocatedFromStoragePool
------------------------------

.. autoclass:: LMI_LVAllocatedFromStoragePool
    :members:

"""

from lmi.storage.BaseProvider import BaseProvider
import pywbem
import blivet
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_LVAllocatedFromStoragePool(BaseProvider):
    """
        Implementation of LMI_LVAllocatedFromStoragePool class.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LVAllocatedFromStoragePool, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'Dependent': None, 'Antecedent': None})

        for device in self.storage.lvs:
            lvprovider = self.provider_manager.get_provider_for_device(device)
            if not lvprovider:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Cannot find provider for device " + device.path)
            vg = device.vg
            vgprovider = self.provider_manager.get_provider_for_device(vg)
            if not vgprovider:
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                        "Cannot find provider for device " + vg.path)

            model['Dependent'] = lvprovider.get_name_for_device(device)
            model['Antecedent'] = vgprovider.get_name_for_device(vg)
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, device, vg)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, device=None, vg=None):
        """
            Provider implementation of GetInstance intrinsic method.
            It just checks if Dependent and Antecedent are related.
        """
        if not device:
            device = self.provider_manager.get_device_for_name(
                    model['Dependent'])
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Dependent device")

        if not vg:
            vg = self.provider_manager.get_device_for_name(
                    model['Antecedent'])
        if not vg:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find Antecedent device")

        if not isinstance(device,
                    blivet.devices.LVMLogicalVolumeDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Dependend device is not logical volume: " + device.path)
        if not isinstance(vg, blivet.devices.LVMVolumeGroupDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Antecedent device is not volume vroup: " + vg.path)
        if vg != device.vg:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Antecedent is not related to Dependent device")

        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_StorageExtent",
                "CIM_StoragePool")
